#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2023, Tim Cragg (@timcragg)
# Copyright: (c) 2023, Akini Ross (@akinross)
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: aci_access_switch_policy_group
short_description: Manage Access Switch Policy Groups (infra:AccNodePGrp and infra:SpineAccNodePGrp).
description:
- Manage Access Switch Policy Groups on Cisco ACI fabrics.
options:
  name:
    description:
    - The name of the access switch policy group.
    aliases: [ policy_group ]
    type: str
  description:
    description:
    - The description of the access switch policy group.
    type: str
  switch_type:
    description:
    - Whether this is a leaf or spine policy group
    type: str
    choices: [ leaf, spine ]
    required: true
  spanning_tree_policy:
    description:
    - The spanning tree policy bound to the access switch policy group.
    - Only available in APIC version 5.2 or later.
    - Only available when I(switch_type=leaf).
    - The APIC defaults to C("") which results in the target DN set to the default policy when unset during creation.
    type: str
  bfd_ipv4_policy:
    description:
    - The BFD IPv4 policy bound to the access switch policy group.
    - The APIC defaults to C("") which results in the target DN set to the default policy when unset during creation.
    type: str
  bfd_ipv6_policy:
    description:
    - The BFD IPv6 policy bound to the access switch policy group.
    - The APIC defaults to C("") which results in the target DN set to the default policy when unset during creation.
    type: str
  bfd_multihop_ipv4_policy:
    description:
    - The BFD multihop IPv4 policy bound to the access switch policy group.
    - Only available in APIC version 5.x or later.
    - Only available when I(switch_type=leaf).
    - The APIC defaults to C("") which results in the target DN set to the default policy when unset during creation.
    type: str
  bfd_multihop_ipv6_policy:
    description:
    - The BFD multihop IPv6 policy bound to the access switch policy group.
    - Only available in APIC version 5.x or later.
    - Only available when I(switch_type=leaf).
    - The APIC defaults to C("") which results in the target DN set to the default policy when unset during creation.
    type: str
  fibre_channel_node_policy:
    description:
    - The fibre channel node policy bound to the access switch policy group.
    - Only available when I(switch_type=leaf).
    - The APIC defaults to C("") which results in the target DN set to the default policy when unset during creation.
    type: str
  poe_node_policy:
    description:
    - The PoE node policy bound to the access switch policy group.
    - Only available when I(switch_type=leaf).
    - The APIC defaults to C("") which results in the target DN set to the default policy when unset during creation.
    type: str
  fibre_channel_san_policy:
    description:
    - The fibre channel SAN policy bound to the access switch policy group.
    - Only available when I(switch_type=leaf).
    - The APIC defaults to C("") which results in the target DN set to the default policy when unset during creation.
    type: str
  monitoring_policy:
    description:
    - The monitoring policy bound to the access switch policy group.
    - Only available when I(switch_type=leaf).
    - The APIC defaults to C("") which results in the target DN set to the default policy when unset during creation.
    type: str
  netflow_node_policy:
    description:
    - The netflow node policy bound to the access switch policy group.
    - Only available when I(switch_type=leaf).
    - The APIC defaults to C("") which results in the target DN set to the default policy when unset during creation.
    type: str
  copp_policy:
    description:
    - The CoPP policy bound to the access switch policy group.
    - The APIC defaults to C("") which results in the target DN set to the default policy when unset during creation.
    type: str
  forward_scale_profile_policy:
    description:
    - The forward scale profile policy bound to the access switch policy group.
    - Only available when I(switch_type=leaf).
    - The APIC defaults to C("") which results in the target DN set to the default policy when unset during creation.
    type: str
  fast_link_failover_policy:
    description:
    - The fast link failover policy bound to the access switch policy group.
    - Only available when I(switch_type=leaf).
    - The APIC defaults to C("") which results in the target DN set to the default policy when unset during creation.
    type: str
  node_802_1x_authentication_policy:
    description:
    - The 802.1x node authentication policy bound to the access switch policy group.
    - Only available when I(switch_type=leaf).
    - The APIC defaults to C("") which results in the target DN set to the default policy when unset during creation.
    type: str
  copp_pre_filter_policy:
    description:
    - The CoPP pre-filter policy bound to the access switch policy group.
    - The APIC defaults to C("") which results in the target DN set to the default policy when unset during creation.
    type: str
  equipment_flash_policy:
    description:
    - The equipment flash policy bound to the access switch policy group.
    - Only available when I(switch_type=leaf).
    - The APIC defaults to C("") which results in the target DN set to the default policy when unset during creation.
    type: str
  cdp_policy:
    description:
    - The CDP policy bound to the access switch policy group.
    - The APIC defaults to C("") which results in the target DN set to the default policy when unset during creation.
    type: str
  lldp_policy:
    description:
    - The LLDP policy bound to the access switch policy group.
    - The APIC defaults to C("") which results in the target DN set to the default policy when unset during creation.
    type: str
  sync_e_node_policy:
    description:
    - The SyncE node policy bound to the access switch policy group.
    - Only available in APIC version 5.x or later.
    - Only available when I(switch_type=leaf).
    - The APIC defaults to C("") which results in the target DN set to the default policy when unset during creation.
    type: str
  ptp_node_policy:
    description:
    - The PTP node policy bound to the access switch policy group.
    - Only available in APIC version 5.2 or later.
    - Only available when I(switch_type=leaf).
    - The APIC defaults to C("") which results in the target DN set to the default policy when unset during creation.
    type: str
  usb_configuration_policy:
    description:
    - The USB configuration policy bound to the access switch policy group.
    - Only available in APIC version 5.2 or later.
    - The APIC defaults to C("") which results in the target DN set to the default policy when unset during creation.
    type: str
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

seealso:
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(infra:AccNodePGrp) and B(infra:SpineAccNodePGrp).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Tim Cragg (@timcragg)
- Akini Ross (@akinross)
"""

EXAMPLES = r"""
- name: Create Leaf Access Switch Policy Group
  cisco.aci.aci_access_switch_policy_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    name: ansible_pol_grp_spine
    switch_type: leaf
    spanning_tree_policy: example_spanning_tree_policy
    bfd_ipv4_policy: example_bfd_ipv4_policy
    bfd_ipv6_policy: example_bfd_ipv6_policy
    fibre_channel_node_policy: example_fibre_channel_node_policy
    poe_node_policy: example_poe_node_policy
    fibre_channel_san_policy: example_fibre_channel_san_policy
    monitoring_policy: example_monitoring_policy
    copp_policy: example_copp_policy
    forward_scale_profile_policy: example_forward_scale_profile_policy
    fast_link_failover_policy: example_fast_link_failover_policy
    node_802_1x_authentication_policy: example_node_802_1x_authentication_policy
    copp_pre_filter_policy: example_copp_pre_filter_policy
    equipment_flash_policy: example_equipment_flash_policy
    cdp_policy: example_cdp_policy
    lldp_policy: example_lldp_policy
    state: present
  delegate_to: localhost

- name: Create Spine Access Switch Policy Group
  cisco.aci.aci_access_switch_policy_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    name: ansible_pol_grp_leaf
    switch_type: spine
    bfd_ipv4_policy: example_bfd_ipv4_policy
    bfd_ipv6_policy: example_bfd_ipv6_policy
    copp_policy: example_copp_policy
    copp_pre_filter_policy: example_copp_pre_filter_policy
    cdp_policy: example_cdp_policy
    lldp_policy: example_lldp_policy
    state: present
  delegate_to: localhost

- name: Delete Leaf Access Switch Policy Group
  cisco.aci.aci_access_switch_policy_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    name: ansible_pol_grp_leaf
    switch_type: leaf
    state: absent
  delegate_to: localhost

- name: Query Leaf Access Switch Policy Group
  cisco.aci.aci_access_switch_policy_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    name: ansible_pol_grp_leaf
    switch_type: leaf
    state: query
  delegate_to: localhost
  register: query_result

- name: Query All Leaf Access Switch Policy Groups
  cisco.aci.aci_access_switch_policy_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    switch_type: leaf
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r"""
   current:
     description: The existing configuration from the APIC after the module has finished
     returned: success
     type: list
     sample:
       [
           {
               "fvTenant": {
                   "attributes": {
                       "descr": "Production environment",
                       "dn": "uni/tn-production",
                       "name": "production",
                       "nameAlias": "",
                       "ownerKey": "",
                       "ownerTag": ""
                   }
               }
           }
       ]
   error:
     description: The error information as returned from the APIC
     returned: failure
     type: dict
     sample:
       {
           "code": "122",
           "text": "unknown managed object class foo"
       }
   raw:
     description: The raw output returned by the APIC REST API (xml or json)
     returned: parse error
     type: str
     sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class "/></imdata>'
   sent:
     description: The actual/minimal configuration pushed to the APIC
     returned: info
     type: list
     sample:
       {
           "fvTenant": {
               "attributes": {
                   "descr": "Production environment"
               }
           }
       }
   previous:
     description: The original configuration from the APIC before the module has started
     returned: info
     type: list
     sample:
       [
           {
               "fvTenant": {
                   "attributes": {
                       "descr": "Production",
                       "dn": "uni/tn-production",
                       "name": "production",
                       "nameAlias": "",
                       "ownerKey": "",
                       "ownerTag": ""
                   }
               }
           }
       ]
   proposed:
     description: The assembled configuration from the user-provided parameters
     returned: info
     type: dict
     sample:
       {
           "fvTenant": {
               "attributes": {
                   "descr": "Production environment",
                   "name": "production"
               }
           }
       }
   filter_string:
     description: The filter string used for the request
     returned: failure or debug
     type: str
     sample: ?rsp-prop-include=config-only
   method:
     description: The HTTP method used for the request to the APIC
     returned: failure or debug
     type: str
     sample: POST
   response:
     description: The HTTP response from the APIC
     returned: failure or debug
     type: str
     sample: OK (30 bytes)
   status:
     description: The HTTP status from the APIC
     returned: failure or debug
     type: int
     sample: 200
   url:
     description: The HTTP url used for the request to the APIC
     returned: failure or debug
     type: str
     sample: https://10.11.12.13/api/mo/uni/tn-production.json
   """

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_owner_spec
from ansible_collections.cisco.aci.plugins.module_utils.constants import ACI_ACCESS_SWITCH_POLICY_GROUP_CLASS_MAPPING


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        name=dict(type="str", aliases=["policy_group"]),
        description=dict(type="str"),
        switch_type=dict(type="str", choices=["leaf", "spine"], required=True),
        spanning_tree_policy=dict(type="str"),
        bfd_ipv4_policy=dict(type="str"),
        bfd_ipv6_policy=dict(type="str"),
        bfd_multihop_ipv4_policy=dict(type="str"),
        bfd_multihop_ipv6_policy=dict(type="str"),
        fibre_channel_node_policy=dict(type="str"),
        poe_node_policy=dict(type="str"),
        fibre_channel_san_policy=dict(type="str"),
        monitoring_policy=dict(type="str"),
        netflow_node_policy=dict(type="str"),
        copp_policy=dict(type="str"),
        forward_scale_profile_policy=dict(type="str"),
        fast_link_failover_policy=dict(type="str"),
        node_802_1x_authentication_policy=dict(type="str"),
        copp_pre_filter_policy=dict(type="str"),
        equipment_flash_policy=dict(type="str"),
        cdp_policy=dict(type="str"),
        lldp_policy=dict(type="str"),
        sync_e_node_policy=dict(type="str"),
        ptp_node_policy=dict(type="str"),
        usb_configuration_policy=dict(type="str"),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["name"]],
            ["state", "present", ["name"]],
        ],
    )
    name = module.params.get("name")
    description = module.params.get("description")
    switch_type = module.params.get("switch_type")
    spanning_tree_policy = module.params.get("spanning_tree_policy")
    bfd_ipv4_policy = module.params.get("bfd_ipv4_policy")
    bfd_ipv6_policy = module.params.get("bfd_ipv6_policy")
    bfd_multihop_ipv4_policy = module.params.get("bfd_multihop_ipv4_policy")
    bfd_multihop_ipv6_policy = module.params.get("bfd_multihop_ipv6_policy")
    fibre_channel_node_policy = module.params.get("fibre_channel_node_policy")
    poe_node_policy = module.params.get("poe_node_policy")
    fibre_channel_san_policy = module.params.get("fibre_channel_san_policy")
    monitoring_policy = module.params.get("monitoring_policy")
    netflow_node_policy = module.params.get("netflow_node_policy")
    copp_policy = module.params.get("copp_policy")
    forward_scale_profile_policy = module.params.get("forward_scale_profile_policy")
    fast_link_failover_policy = module.params.get("fast_link_failover_policy")
    node_802_1x_authentication_policy = module.params.get("node_802_1x_authentication_policy")
    copp_pre_filter_policy = module.params.get("copp_pre_filter_policy")
    equipment_flash_policy = module.params.get("equipment_flash_policy")
    cdp_policy = module.params.get("cdp_policy")
    lldp_policy = module.params.get("lldp_policy")
    sync_e_node_policy = module.params.get("sync_e_node_policy")
    ptp_node_policy = module.params.get("ptp_node_policy")
    usb_configuration_policy = module.params.get("usb_configuration_policy")
    state = module.params.get("state")

    aci = ACIModule(module)

    if switch_type == "spine" and not all(
        v is None
        for v in [
            spanning_tree_policy,
            bfd_multihop_ipv4_policy,
            bfd_multihop_ipv6_policy,
            fibre_channel_node_policy,
            poe_node_policy,
            fibre_channel_san_policy,
            monitoring_policy,
            netflow_node_policy,
            forward_scale_profile_policy,
            fast_link_failover_policy,
            node_802_1x_authentication_policy,
            equipment_flash_policy,
            sync_e_node_policy,
            ptp_node_policy,
        ]
    ):
        aci.fail_json(msg="Unsupported policy provided for spine switch type.")

    class_name = ACI_ACCESS_SWITCH_POLICY_GROUP_CLASS_MAPPING.get(switch_type).get("class_name")

    aci.construct_url(
        root_class=dict(
            aci_class=class_name,
            aci_rn=ACI_ACCESS_SWITCH_POLICY_GROUP_CLASS_MAPPING.get(switch_type).get("rn").format(name),
            module_object=name,
            target_filter={"name": name},
        ),
        rsp_subtree="children",
    )

    aci.get_existing()

    if state == "present":
        child_configs = []
        if forward_scale_profile_policy is not None:
            child_configs.append({"infraRsTopoctrlFwdScaleProfPol": {"attributes": {"tnTopoctrlFwdScaleProfilePolName": forward_scale_profile_policy}}})
        if usb_configuration_policy is not None:
            child_configs.append(
                {
                    ACI_ACCESS_SWITCH_POLICY_GROUP_CLASS_MAPPING.get(switch_type)
                    .get("usb_configuration_policy")
                    .get("class_name"): {
                        "attributes": {
                            ACI_ACCESS_SWITCH_POLICY_GROUP_CLASS_MAPPING.get(switch_type)
                            .get("usb_configuration_policy")
                            .get("tn_name"): usb_configuration_policy
                        }
                    }
                }
            )
        if lldp_policy is not None:
            child_configs.append(
                {
                    ACI_ACCESS_SWITCH_POLICY_GROUP_CLASS_MAPPING.get(switch_type)
                    .get("lldp_policy")
                    .get("class_name"): {
                        "attributes": {ACI_ACCESS_SWITCH_POLICY_GROUP_CLASS_MAPPING.get(switch_type).get("lldp_policy").get("tn_name"): lldp_policy}
                    }
                }
            )
        if cdp_policy is not None:
            child_configs.append(
                {
                    ACI_ACCESS_SWITCH_POLICY_GROUP_CLASS_MAPPING.get(switch_type)
                    .get("cdp_policy")
                    .get("class_name"): {
                        "attributes": {ACI_ACCESS_SWITCH_POLICY_GROUP_CLASS_MAPPING.get(switch_type).get("cdp_policy").get("tn_name"): cdp_policy}
                    }
                }
            )
        if bfd_ipv4_policy is not None:
            child_configs.append(
                {
                    ACI_ACCESS_SWITCH_POLICY_GROUP_CLASS_MAPPING.get(switch_type)
                    .get("bfd_ipv4_policy")
                    .get("class_name"): {
                        "attributes": {ACI_ACCESS_SWITCH_POLICY_GROUP_CLASS_MAPPING.get(switch_type).get("bfd_ipv4_policy").get("tn_name"): bfd_ipv4_policy}
                    }
                }
            )
        if bfd_ipv6_policy is not None:
            child_configs.append(
                {
                    ACI_ACCESS_SWITCH_POLICY_GROUP_CLASS_MAPPING.get(switch_type)
                    .get("bfd_ipv6_policy")
                    .get("class_name"): {
                        "attributes": {ACI_ACCESS_SWITCH_POLICY_GROUP_CLASS_MAPPING.get(switch_type).get("bfd_ipv6_policy").get("tn_name"): bfd_ipv6_policy}
                    }
                }
            )
        if sync_e_node_policy is not None:
            child_configs.append({"infraRsSynceInstPol": {"attributes": {"tnSynceInstPolName": sync_e_node_policy}}})
        if poe_node_policy is not None:
            child_configs.append({"infraRsPoeInstPol": {"attributes": {"tnPoeInstPolName": poe_node_policy}}})
        if bfd_multihop_ipv4_policy is not None:
            child_configs.append({"infraRsBfdMhIpv4InstPol": {"attributes": {"tnBfdMhIpv4InstPolName": bfd_multihop_ipv4_policy}}})
        if bfd_multihop_ipv6_policy is not None:
            child_configs.append({"infraRsBfdMhIpv6InstPol": {"attributes": {"tnBfdMhIpv6InstPolName": bfd_multihop_ipv6_policy}}})
        if equipment_flash_policy is not None:
            child_configs.append({"infraRsEquipmentFlashConfigPol": {"attributes": {"tnEquipmentFlashConfigPolName": equipment_flash_policy}}})
        if monitoring_policy is not None:
            child_configs.append({"infraRsMonNodeInfraPol": {"attributes": {"tnMonInfraPolName": monitoring_policy}}})
        if fibre_channel_node_policy is not None:
            child_configs.append({"infraRsFcInstPol": {"attributes": {"tnFcInstPolName": fibre_channel_node_policy}}})
        if fast_link_failover_policy is not None:
            child_configs.append(
                {"infraRsTopoctrlFastLinkFailoverInstPol": {"attributes": {"tnTopoctrlFastLinkFailoverInstPolName": fast_link_failover_policy}}}
            )
        if spanning_tree_policy is not None:
            child_configs.append({"infraRsMstInstPol": {"attributes": {"tnStpInstPolName": spanning_tree_policy}}})
        if fibre_channel_san_policy is not None:
            child_configs.append({"infraRsFcFabricPol": {"attributes": {"tnFcFabricPolName": fibre_channel_san_policy}}})
        if copp_policy is not None:
            child_configs.append(
                {
                    ACI_ACCESS_SWITCH_POLICY_GROUP_CLASS_MAPPING.get(switch_type)
                    .get("copp_policy")
                    .get("class_name"): {
                        "attributes": {ACI_ACCESS_SWITCH_POLICY_GROUP_CLASS_MAPPING.get(switch_type).get("copp_policy").get("tn_name"): copp_policy}
                    }
                }
            )
        if node_802_1x_authentication_policy is not None:
            child_configs.append({"infraRsL2NodeAuthPol": {"attributes": {"tnL2NodeAuthPolName": node_802_1x_authentication_policy}}})
        if copp_pre_filter_policy is not None:
            child_configs.append(
                {
                    ACI_ACCESS_SWITCH_POLICY_GROUP_CLASS_MAPPING.get(switch_type)
                    .get("copp_pre_filter_policy")
                    .get("class_name"): {
                        "attributes": {
                            ACI_ACCESS_SWITCH_POLICY_GROUP_CLASS_MAPPING.get(switch_type).get("copp_pre_filter_policy").get("tn_name"): copp_pre_filter_policy
                        }
                    }
                }
            )
        if netflow_node_policy is not None:
            child_configs.append({"infraRsNetflowNodePol": {"attributes": {"tnNetflowNodePolName": netflow_node_policy}}})
        if ptp_node_policy is not None:
            child_configs.append({"infraRsPtpInstPol": {"attributes": {"tnPtpInstPolName": ptp_node_policy}}})

        if child_configs == []:
            child_configs = None

        aci.payload(
            aci_class=class_name,
            class_config=dict(
                descr=description,
            ),
            child_configs=child_configs,
        )

        aci.get_diff(aci_class=class_name)

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2019, RusoSova
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r'''
---
module: win_computer_description
short_description: Set windows description, owner and organization
description:
 - This module sets Windows description that is shown under My Computer properties. Module also sets
   Windows license owner and organization. License information can be viewed by running winver commad.
version_added: 2.7.0
options:
 description:
   description:
     - String value to apply to Windows descripton. Specify value of "" to clear the value.
   required: false
   type: str
 organization:
   description:
     - String value of organization that the Windows is licensed to. Specify value of "" to clear the value.
   required: false
   type: str
 owner:
   description:
     - String value of the persona that the Windows is licensed to. Specify value of "" to clear the value.
   required: false
   type: str
author:
 - RusoSova (@RusoSova)
'''

EXAMPLES = r'''
- name: Set Windows description, owner and organization
  ansible.windows.win_computer_description:
   description: Best Box
   owner: RusoSova
   organization: MyOrg
  register: result

- name: Set Windows description only
  ansible.windows.win_computer_description:
   description: This is my Windows machine
  register: result

- name: Set organization and clear owner field
  ansible.windows.win_computer_description:
   owner: ''
   organization: Black Mesa

- name: Clear organization, description and owner
  ansible.windows.win_computer_description:
   organization: ""
   owner: ""
   description: ""
  register: result
'''

RETURN = r'''
#
'''



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The Bgp_neighbor_address_family parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


def _tmpl_aigp(config_data):
    conf = config_data.get("aigp", {})
    commands = []
    if conf:
        if "set" in conf:
            commands.append("aigp")
        if "disable" in conf:
            commands.append("aigp disable")
        if "send_cost_community_disable" in conf:
            commands.append("aigp send cost-community disable")
        if "send_med" in conf and "set" in conf.get("send_med", {}):
            commands.append("aigp send med")
        if "send_med" in conf and "disable" in conf.get("send_med", {}):
            commands.append("aigp send med disable")
    return commands


def _tmpl_validation(config_data):
    conf = config_data.get("validation", {})
    command = ""
    if conf:
        if "set" in conf:
            command = "validation"
        if "disable" in conf:
            command = "validation disbale"
        if "redirect" in conf:
            command = "validation redirect"
    return command


def _tmpl_next_hop_unchanged(config_data):
    conf = config_data.get("next_hop_unchanged", {})
    command = ""
    if conf:
        if "set" in conf:
            command = "next-hop-unchanged"
        if "inheritance_disable" in conf:
            command += "next-hop-unchanged inheritance-disable"
        if "multipath" in conf:
            command = "next-hop-unchanged multipath"
    return command


def _tmpl_maximum_prefix(config_data):
    conf = config_data.get("maximum_prefix", {})
    if conf:
        command = "maximum-prefix"
        if "max_limit" in conf:
            command += " " + str(conf["max_limit"])
        if "threshold_value" in conf:
            command += " " + str(conf["threshold_value"])
        if "restart" in conf:
            command += " restart " + str(conf["restart"])
        elif "warning_only" in conf:
            command += " warning-only"
        elif "discard_extra_paths" in conf:
            command += " discard-extra-paths"

    return command


def _tmpl_soft_reconfiguration(config_data):
    conf = config_data.get("soft_reconfiguration", {})
    if conf:
        command = "soft-reconfiguration "
        if "inbound" in conf:
            command += "inbound"
            if "set" in conf["inbound"]:
                pass
            elif "always" in conf["inbound"]:
                command += " always"
            if "inheritance_disable" in conf["inbound"]:
                command += " inheritance-disable"

    return command


def _tmpl_remove_private_AS(config_data):
    conf = config_data.get("remove_private_AS", {})
    if conf:
        command = " "
        if "set" in conf:
            command = "remove-private-AS"
        if "inbound" in conf:
            command += " inbound"
        if "entire_aspath" in conf:
            command += " entire-aspath"
        elif "inheritance_disable" in conf:
            command = "remove-private-AS inheritance-disable"
    return command


def _tmpl_default_originate(config_data):
    conf = config_data.get("default_originate", {})
    command = ""
    if conf:
        if "set" in conf:
            command = "default-originate"
        if "inheritance_disable" in conf:
            command = "default-originate inheritance-disable"
        if "route_policy" in conf:
            command = "default-originate route_policy " + conf["route_policy"]
    return command


class Bgp_neighbor_address_familyTemplate(NetworkTemplate):
    def __init__(self, lines=None):
        super(Bgp_neighbor_address_familyTemplate, self).__init__(
            lines=lines,
            tmplt=self,
        )

    # fmt: off
    PARSERS = [
        {
            "name": "router",
            "getval": re.compile(
                r"""
                ^router\s
                bgp
                \s(?P<as_num>\S+)
                $""",
                re.VERBOSE,
            ),
            "setval": "router bgp {{ as_number }}",
            "compval": "as_number",
            "result": {"as_number": "{{ as_num }}"},
            "shared": True,
        },
        {
            "name": "address_family",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                (\s+(?P<nbr_address>neighbor\s\S+))
                (?P<address_family>\s+address-family\s(?P<afi>\S+)\s(?P<safi>\S+))
                $""", re.VERBOSE,
            ),
            "setval": "address-family {{ afi}} {{safi}}",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "neighbor_address": "{{nbr_address.split(" ")[1]}}",
                                "address_family": {
                                    '{{"address_family_" + afi + "_" + safi }}': {
                                        "afi": "{{ afi}}",
                                        "safi": "{{safi}}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
            "shared": True,
        },
        {
            "name": "aigp",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+(?P<nbr_address>neighbor\s\S+)
                \saigp(?P<aigp>)
                (\sdisable(?P<disable>))?
                (\ssend\smed(?P<send_med>))?
                (\ssend\smed\sdisable(?P<send_disable>))?
                (\ssend\scost-community\sdisable(?P<cc_disable>))?
                $""", re.VERBOSE,
            ),
            "setval": _tmpl_aigp,
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "address_family": {
                                    '{{"address_family_" + afi + "_" + safi }}': {
                                        "aigp": {
                                            "set": "{{ True if aigp is defined }}",
                                            "disable": "{{ True if disable is defined}}",
                                            "send_med": {
                                                "set": "{{ True if send_med is defined }}",
                                                "disable": "{{ True if send_disable is defined}}",
                                            },
                                            "send_cost_community_disable": "{{True if cc_disable is defined}}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "allowas_in",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+(?P<nbr_address>neighbor\s\S+)
                \sallowas-in(?P<allowas_in>)(\s(?P<value>\S+))?
                $""", re.VERBOSE,
            ),
            "setval": "allowas-in {{allowas_in.value if allowas_in.value is defined }}",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "address_family": {
                                    '{{"address_family_" + afi + "_" + safi }}': {
                                        "allowas_in": {
                                            "set": "{{True if allowas_in is defined and value is not defined}}",
                                            "value": "{{value }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },

        },
        {
            "name": "as_override",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+(?P<nbr_address>neighbor\s\S+)
                \sas-override(?P<as_override>)
                (\sinheritance-disable(?P<inheritance_disable>))?
                $""", re.VERBOSE,
            ),
            "setval": "as-override{{' inheritance-disable' if as_override.inheritance_disable is defined else ''}}",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "address_family": {
                                    '{{"address_family_" + afi + "_" + safi }}': {
                                        "as_override": {
                                            "set": "{{True if as_override is defined "
                                                   "and inheritance_disable is not defined}}",
                                            "inheritance_disable": "{{True if inheritance_disable is defined}}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "bestpath_origin_as_allow_invalid",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \sbestpath\sorigin-as\sallow\sinvalid(?P<invalid>)
                $""", re.VERBOSE,
            ),
            "setval": "bestpath origin-as allow invalid",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "address_family": {
                                    '{{"address_family_" + afi + "_" + safi }}': {
                                        "bestpath_origin_as_allow_invalid": "{{ True if invalid is defined}}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "capability_orf_prefix",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+(?P<nbr_address>neighbor\s\S+)
                \scapability\sorf\sprefix\s(?P<capability_orf_prefix>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "capability orf prefix {{capability_orf_prefix }}",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "address_family": {
                                    '{{"address_family_" + afi + "_" + safi }}': {
                                        "capability_orf_prefix": "{{capability_orf_prefix}}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "default_originate",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+(?P<nbr_address>neighbor\s\S+)
                \sdefault-originate(?P<default_originate>)
                (\sroute-policy\s(?P<route_policy>\S+))?
                (\sinheritance-disable(?P<inheritance_disable>))?
                $""", re.VERBOSE,
            ),
            "setval": _tmpl_default_originate,
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "address_family": {
                                    '{{"address_family_" + afi + "_" + safi}}': {
                                        "default_originate": {
                                            "set": "{{True if default_originate is defined}}",
                                            "route_policy": "{{route_policy}}",
                                            "inheritance_disable": "{{True if inheritance_disable is defined}}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "long_lived_graceful_restart_capable",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+(?P<nbr_address>neighbor\s\S+)
                \s+long-lived-graceful-restart
                \s(?P<capable>capable)
                $""", re.VERBOSE,
            ),
            "setval": "long-lived-graceful-restart capable",
            "compval": "long_lived_graceful_restart.capable",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "address_family": {
                                    '{{"address_family_" + afi + "_" + safi }}': {
                                        "long_lived_graceful_restart": {
                                            "capable": "{{True if capable is defined}}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "long_lived_graceful_restart_stale_time",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+(?P<nbr_address>neighbor\s\S+)
                \s+long-lived-graceful-restart
                \s+stale-time\ssend\s(?P<stale_time_send>\d+)\saccept\s(?P<accept>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "long-lived-graceful-restart stale-time send "
                      "{{stale_time.send}} accept {{stale_time.accept}}",
            "compval": "long_lived_graceful_restart.stale_time",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "address_family": {
                                    '{{"address_family_" + afi + "_" + safi }}': {
                                        "long_lived_graceful_restart": {
                                            "stale_time": {
                                                "send": "{{stale_time_send}}",
                                                "accept": "{{accept}}",
                                            },
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "maximum_prefix",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+(?P<nbr_address>neighbor\s\S+)
                \s+maximum-prefix
                (\s(?P<maximum_prefix>\d+))?
                (\s(?P<threshold_value>\d+))?
                (\srestart\s(?P<restart>\d+))?
                (\swarning-only\s(?P<warning_only>))?
                (\sdiscard-extra-paths\s(?P<discard_extra_paths>))?
                $""", re.VERBOSE,
            ),
            "setval": _tmpl_maximum_prefix,
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "address_family": {
                                    '{{"address_family_" + afi + "_" + safi }}': {
                                        "maximum_prefix": {
                                            "max_limit": "{{maximum_prefix}}",
                                            "threshold_value": "{{threshold_value}}",
                                            "restart": "{{restart}}",
                                            "warning_only": "{{ True if warning_only is defined}}",
                                            "discard_extra_paths": "{{ True if discard_extra_paths is defined}}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "multipath",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+(?P<nbr_address>neighbor\s\S+)
                \smultipath(?P<multipath>)
                $""", re.VERBOSE,
            ),
            "setval": "multipath",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "address_family": {
                                    '{{"address_family_" + afi + "_" + safi }}': {
                                        "multipath": "{{True if multipath is defined}}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "next_hop_self",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+(?P<nbr_address>neighbor\s\S+)
                \snext-hop-self(?P<next_hop_self>)
                (\sinheritance-disable(?P<inheritance_disable>))?
                $""", re.VERBOSE,
            ),
            "setval": "next-hop-self{{' inheritance-disable' if next_hop_self.inheritance_disable is defined else ''}}",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "address_family": {
                                    '{{"address_family_" + afi + "_" + safi }}': {
                                        "next_hop_self": {
                                            "set": "{{True if next_hop_self is defined and"
                                                   " inheritance_disable is not defined}}",
                                            "inheritance_disable": "{{True if inheritance_disable is defined}}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "next_hop_unchanged",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+(?P<nbr_address>neighbor\s\S+)
                \snext-hop-unchanged(?P<next_hop_unchanged>)
                (\sinheritance-disable(?P<inheritance_disable>))?
                (\smultipath(?P<multipath>))?
                $""", re.VERBOSE,
            ),
            "setval": _tmpl_next_hop_unchanged,
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "address_family": {
                                    '{{"address_family_" + afi + "_" + safi}}': {
                                        "next_hop_unchanged": {
                                            "set": "{{True if next_hop_self is defined }}",
                                            "inheritance_disable": "{{True if inheritance_disable is defined}}",
                                            "multipath": "{{True if multipath is defined}}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "optimal_route_reflection_group_name",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+(?P<nbr_address>neighbor\s\S+)
                \soptimal-route-reflection\s(?P<group_name>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "optimal-route-reflection {{optimal_route_reflection_group_name}}",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "address_family": {
                                    '{{"address_family_" + afi + "_" + safi }}': {
                                        "optimal_route_reflection_group_name": "{{ group_name}}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "orf_route_policy",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+(?P<nbr_address>neighbor\s\S+)
                \sorf\sroute-policy\s(?P<orf_rr>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "orf route-policy {{orf_route_policy}}",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "address_family": {
                                    '{{"address_family_" + afi + "_" + safi }}': {
                                        "orf_route_policy": "{{orf_rr}}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "origin_as",
            "getval": re.compile(
                r"""
                \s+(?P<nbr_address>neighbor\s\S+)
                \sorigin-as\svalidation\sdisable(?P<origin_as>)
                $""", re.VERBOSE,
            ),
            "setval": "origin-as validation disable",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "address_family": {
                                    '{{"address_family_" + afi + "_" + safi }}': {
                                        "origin_as": {
                                            "validation": {
                                                "disable": "{{True if origin_as is defined }}",
                                            },
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "route_policy.inbound",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+(?P<nbr_address>neighbor\s\S+)
                \sroute-policy\s(?P<route_policy>\S+)
                \sin
                $""", re.VERBOSE,
            ),
            "setval": "route-policy {{route_policy.inbound}} in",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "address_family": {
                                    '{{"address_family_" + afi + "_" + safi }}': {
                                        "route_policy": {
                                            "inbound": "{{route_policy}}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "route_policy.outbound",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+(?P<nbr_address>neighbor\s\S+)
                \sroute-policy\s(?P<route_policy>\S+)
                \sout
                $""", re.VERBOSE,
            ),
            "setval": "route-policy {{route_policy.outbound}} out",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "address_family": {
                                    '{{"address_family_" + afi + "_" + safi }}': {
                                        "route_policy": {
                                            "outbound": "{{route_policy}}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "remove_private_AS",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+(?P<nbr_address>neighbor\s\S+)
                \sremove-private-AS(?P<remove_private_AS>)
                (\sinbound(?P<inbound>))?
                (\sentire-aspath(?P<entire_aspath>))?
                (\sinheritance-disable(?P<inheritance_disable>))?
                $""", re.VERBOSE,
            ),
            "setval": _tmpl_remove_private_AS,
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "address_family": {
                                    '{{"address_family_" + afi + "_" + safi }}': {
                                        "remove_private_AS": {
                                            "set": "{{True if remove_private_AS is defined}}",
                                            "inbound": "{{True if inbound is defined}}",
                                            "entire_aspath": "{{True if entire_aspath is defined}}",
                                            "inheritance_disable": "{{True if inheritance_disable is defined}}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "route_reflector_client",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+(?P<nbr_address>neighbor\s\S+)
                \sroute-reflector-client(?P<route_reflector_client>)
                (\sinheritance-disable(?P<inheritance_disable>))?
                $""", re.VERBOSE,
            ),
            "setval": "route-reflector-client{{' inheritance-disable' "
                      "if route_reflector_client.inheritance_disable is defined }}",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "address_family": {
                                    '{{"address_family_" + afi + "_" + safi }}': {
                                        "route_reflector_client": {
                                            "set": "{{True if route_reflector_client is defined and "
                                                   "inheritance_disable is not defined }}",
                                            "inheritance_disable": "{{True if inheritance_disable is defined}}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "send_community_ebgp",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+(?P<nbr_address>neighbor\s\S+)
                \ssend-community-ebgp(?P<send_community_ebgp>)
                (\sinheritance-disable(?P<inheritance_disable>))?
                $""", re.VERBOSE,
            ),
            "setval": "send-community-ebgp{{' inheritance-disable' "
                      "if send_community_ebgp.inheritance_disable is defined else ''}}",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "address_family": {
                                    '{{"address_family_" + afi + "_" + safi }}': {
                                        "send_community_ebgp": {
                                            "set": "{{True if send_community_ebgp is defined and "
                                                   "inheritance_disable is not defined}}",
                                            "inheritance_disable": "{{True if inheritance_disable is defined}}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "send_community_gshut_ebgp",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+(?P<nbr_address>neighbor\s\S+)
                \ssend-community-gshut-ebgp(?P<send_community_gshut_ebg>)
                (\sinheritance-disable(?P<inheritance_disable>))?
                $""", re.VERBOSE,
            ),
            "setval": "send-community-gshut-ebgp{{' inheritance-disable' "
                      "if send_community_gshut_ebgp.inheritance_disable is defined else ''}}",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "address_family": {
                                    '{{"address_family_" + afi + "_" + safi }}': {
                                        "send_community_gshut_ebgp": {
                                            "set": "{{True if send_community_gshut_ebg is defined and "
                                                   "inheritance_disable is not defined}}",
                                            "inheritance_disable": "{{True if inheritance_disable is defined}}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "send_extended_community_ebgp",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+(?P<nbr_address>neighbor\s\S+)
                \ssend-extended-community-ebgp(?P<send_extended_community_ebgp>)
                (\sinheritance-disable(?P<inheritance_disable>))?
                $""", re.VERBOSE,
            ),
            "setval": "send-extended-community-ebgp{{' inheritance-disable' "
                      "if send_extended_community_ebgp.inheritance_disable is defined else ''}}",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "address_family": {
                                    '{{"address_family_" + afi + "_" + safi }}': {
                                        "send_extended_community_ebgp": {
                                            "set": "{{True if send_extended_community_ebgp is defined and "
                                            "inheritance_disable is not defined}}",
                                            "inheritance_disable": "{{True if inheritance_disable is defined}}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "send_multicast_attributes",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+(?P<nbr_address>neighbor\s\S+)
                \s+(?P<send_multicast_attributes>send-multicast-attributes)
                (\sdisable(?P<disable>))?
                $""", re.VERBOSE,
            ),
            "setval": "send-multicast-attributes{{' disable' "
                      "if send_multicast_attributes.disable is defined else ''}}",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "address_family": {
                                    '{{"address_family_" + afi + "_" + safi }}': {
                                        "send_multicast_attributes": {
                                            "set": "{{True if send_multicast_attributes is "
                                                   "defined and disable is not defined}}",
                                            "disable": "{{True if disable is defined}}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "soft_reconfiguration",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+(?P<nbr_address>neighbor\s\S+)
                \ssoft-reconfiguration
                \sinbound(?P<inbound>)
                (\salways(?P<always>))?
                (\sinheritance-disable(?P<inheritance_disable>))?
                $""", re.VERBOSE,
            ),
            "setval": _tmpl_soft_reconfiguration,
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "address_family": {
                                    '{{"address_family_" + afi + "_" + safi }}': {
                                        "soft_reconfiguration": {
                                            "inbound": {
                                                "set": "{{True if inbound is defined and "
                                                       "inheritance_disable is not defined and "
                                                       "always is not defined}}",
                                                "always": "{{True if always is defined }}",
                                                "inheritance_disable": "{{True if inheritance_disable is defined}}",
                                            },
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "weight",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+(?P<nbr_address>neighbor\s\S+)
                \sweight\s(?P<weight>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "weight {{weight}}",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "address_family": {
                                    '{{"address_family_" + afi + "_" + safi }}': {
                                        "weight": "{{weight}}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "validation",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+(?P<nbr_address>neighbor\s\S+)
                \svalidation(?P<validation>)
                (\sredirect(?P<redirect>))?
                 (\sdisable(?P<disable>))?
                $""", re.VERBOSE,
            ),
            "setval": _tmpl_validation,
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "address_family": {
                                    '{{"address_family_" + afi + "_" + safi }}': {
                                        "validation": {
                                            "set": "{{True if validation is defined}}",
                                            "redirect": "{{True if redirect is defined }}",
                                            "disable": "{{ True if disable is defined}}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "site_of_origin",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+(?P<nbr_address>neighbor\s\S+)
                \ssite-of-origin\s(?P<site_of_origin>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "site-of-origin {{site_of_origin}}",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{nbr_address.split(" ")[1]}}": {
                                "address_family": {
                                    '{{"address_family_" + afi + "_" + safi }}': {
                                        "site_of_origin": "{{site_of_origin}}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
    ]
    # fmt: on



#
# -*- coding: utf-8 -*-
# Copyright 2024 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The nxos_vrf_address_family config file.
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to its desired end-state is
created.
"""

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module import (
    ResourceModule,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_merge,
)

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.facts.facts import Facts
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.rm_templates.vrf_address_family import (
    Vrf_address_familyTemplate,
)


class Vrf_address_family(ResourceModule):
    """
    The nxos_vrf_address_family config class
    """

    def __init__(self, module):
        super(Vrf_address_family, self).__init__(
            empty_fact_val={},
            facts_module=Facts(module),
            module=module,
            resource="vrf_address_family",
            tmplt=Vrf_address_familyTemplate(),
        )
        self.parsers = [
            "maximum",
        ]
        self.list_parsers = [
            "route_target.import",
            "route_target.export",
            "export.map",
            "export.vrf",
            "import.map",
            "import.vrf",
        ]

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        if self.state not in ["parsed", "gathered"]:
            self.generate_commands()
            self.run_commands()
        return self.result

    def generate_commands(self):
        """Generate configuration commands to send based on
        want, have and desired state.
        """
        wantd = self._vrf_address_family_list_to_dict(self.want)
        haved = self._vrf_address_family_list_to_dict(self.have)

        # if state is merged, merge want onto have and then compare
        if self.state == "merged":
            wantd = dict_merge(haved, wantd)

        # if state is deleted, empty out wantd and set haved to wantd
        if self.state == "deleted":
            haved = self._filter_have_to_want(haved, wantd)
            wantd = {}

        # remove superfluous config for overridden and deleted
        if self.state in ["overridden", "deleted"]:
            for k, have in haved.items():
                if k not in wantd:
                    self._compare(want={}, have=have, vrf=k)

        if self.state == "purged":
            purge_list = wantd or haved
            for k, item in purge_list.items():
                self.purge(k, item)
        else:
            for k, want in wantd.items():
                self._compare(want=want, have=haved.pop(k, {}), vrf=k)

    def _compare(self, want, have, vrf):
        """Leverages the base class `compare()` method and
        populates the list of commands to be run by comparing
        the `want` and `have` data with the `parsers` defined
        for the Vrf_address_family network resource.
        """
        begin = len(self.commands)
        self._compare_vrf_afs(want=want, have=have)
        if len(self.commands) != begin:
            self.commands.insert(begin, f"vrf context {vrf}")

    def _compare_vrf_afs(self, want, have):
        """Compare the VRF address families lists.
        :params want: the want VRF dictionary
        :params have: the have VRF dictionary
        """
        waafs = want.get("address_families", {})
        haafs = have.get("address_families", {})

        address_fam_list = [
            ("ipv4", "unicast"),
            ("ipv6", "unicast"),
            ("ipv4", "multicast"),
            ("ipv6", "multicast"),
        ]

        for item in address_fam_list:
            begin = len(self.commands)
            for afk, afv in waafs.items():
                if afv.get("afi", "") == item[0] and afv.get("safi", "") == item[1]:
                    self._compare_single_af(wantaf=afv, haveaf=haafs.pop(afk, {}))
            for afk, afv in haafs.items():
                if afv.get("afi", "") == item[0] and afv.get("safi", "") == item[1]:
                    self._compare_single_af(wantaf={}, haveaf=afv)
            if len(self.commands) != begin:
                self.commands.insert(
                    begin,
                    self._tmplt.render(
                        {"afi": item[0], "safi": item[1]},
                        "address_family",
                        False,
                    ),
                )

    def _compare_single_af(self, wantaf, haveaf):
        """Compare a single address family.
        :params want: the want address family dictionary
        :params have: the have address family dictionary
        """
        self.compare(parsers=self.parsers, want=wantaf, have=haveaf)
        self._compare_af_lists(want=wantaf, have=haveaf)

    def _compare_af_lists(self, want, have):
        """Compare single vrf af list items.
        :params want: the want list item dictionary
        :params have: the have list item dictionary
        """

        for attrib in self.list_parsers:
            parser_split = attrib.split(".")
            wdict = self._convert_to_dict(want.get(parser_split[0], []), parser_split[1])
            hdict = self._convert_to_dict(have.get(parser_split[0], []), parser_split[1])

            for key, entry in wdict.items():
                if entry != hdict.pop(key, {}):
                    self.addcmd(entry, attrib, False)
            # remove remaining items in have for replaced
            for entry in hdict.values():
                self.addcmd(entry, attrib, True)

    def purge(self, vrf, item):
        """Purge the VRF configuration"""
        self.commands.append(f"vrf context {vrf}")
        for i, value in item.get("address_families", {}).items():
            self.commands.append(
                self._tmplt.render(
                    {"afi": value.get("afi"), "safi": value.get("safi")},
                    "address_family",
                    True,
                ),
            )

    def _convert_to_dict(self, vrf_af_item: list, parser_item: str) -> dict:
        """Convert to dict based on parser name.

        :params vrf_af_item: the vrf af item
        :params parser_item: the parser name based on which it needs to be converted
        :returns: A dictionary with items that have the key parser_item
        """
        if not vrf_af_item:
            return {}

        result = {}
        for item in vrf_af_item:
            if parser_item in item:
                if parser_item == "vrf":
                    vrf_item = item.get("vrf", {})
                    key = f"vrf_{vrf_item.get('max_prefix', 'noprefix')}_{vrf_item.get('map_import', 'nomap')}"
                else:
                    key = item[parser_item]
                result[key] = item
        return result

    def _filter_have_to_want(self, haved, wantd):
        if isinstance(haved, dict) and isinstance(wantd, dict):
            filtered = {
                k: self._filter_have_to_want(haved[k], wantd[k]) for k in haved if k in wantd
            }
            return {k: v for k, v in filtered.items() if v not in [None, {}, []]}
        elif isinstance(haved, list) and isinstance(wantd, list):
            filtered_list = []
            for h_item in haved:
                if isinstance(h_item, dict):
                    for w_item in wantd:
                        filtered_item = self._filter_have_to_want(h_item, w_item)
                        if filtered_item not in [None, {}, []]:
                            filtered_list.append(filtered_item)
                            break
                else:
                    if h_item in wantd:
                        filtered_list.append(h_item)
            return filtered_list
        else:
            return haved if haved == wantd else None

    def _vrf_address_family_list_to_dict(self, vrf_af_list: list) -> dict:
        """Convert a list of vrf_address_family dictionaries to a dictionary.

        :param vrf_af_list: A list of vrf_address_family dictionaries.
        :type vrf_af_list: list
        :rtype: dict
        :returns: A dictionary of vrf_address_family dictionaries.
        """

        items = {}
        for af_item in vrf_af_list:
            name = af_item.get("name")
            address_families = af_item.get("address_families", [])
            item = {
                "name": name,
                "address_families": {
                    f"{name}_{af.get('afi')}_{af.get('safi')}": af for af in address_families
                },
            }

            items[name] = item
        return items



#!/usr/bin/python
#
# Copyright (c) Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: jenkins_node
short_description: Manage Jenkins nodes
version_added: 10.0.0
description:
  - Manage Jenkins nodes with Jenkins REST API.
requirements:
  - "python-jenkins >= 0.4.12"
author:
  - Connor Newton (@phyrwork)
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: partial
    details:
      - Check mode is unable to show configuration changes for a node that is not yet present.
  diff_mode:
    support: none
options:
  url:
    description:
      - URL of the Jenkins server.
    default: http://localhost:8080
    type: str
  name:
    description:
      - Name of the Jenkins node to manage.
    required: true
    type: str
  user:
    description:
      - User to authenticate with the Jenkins server.
    type: str
  token:
    description:
      - API token to authenticate with the Jenkins server.
    type: str
  state:
    description:
      - Specifies whether the Jenkins node should be V(present) (created), V(absent) (deleted), V(enabled) (online) or V(disabled)
        (offline).
    default: present
    choices: ['enabled', 'disabled', 'present', 'absent']
    type: str
  num_executors:
    description:
      - When specified, sets the Jenkins node executor count.
    type: int
  labels:
    description:
      - When specified, sets the Jenkins node labels.
    type: list
    elements: str
  offline_message:
    description:
      - Specifies the offline reason message to be set when configuring the Jenkins node state.
      - If O(offline_message) is given and requested O(state) is not V(disabled), an error is raised.
      - Internally O(offline_message) is set using the V(toggleOffline) API, so updating the message when the node is already
        offline (current state V(disabled)) is not possible. In this case, a warning is issued.
    type: str
    version_added: 10.0.0
"""

EXAMPLES = r"""
- name: Create a Jenkins node using token authentication
  community.general.jenkins_node:
    url: http://localhost:8080
    user: jenkins
    token: 11eb751baabb66c4d1cb8dc4e0fb142cde
    name: my-node
    state: present

- name: Set number of executors on Jenkins node
  community.general.jenkins_node:
    name: my-node
    state: present
    num_executors: 4

- name: Set labels on Jenkins node
  community.general.jenkins_node:
    name: my-node
    state: present
    labels:
      - label-1
      - label-2
      - label-3

- name: Set Jenkins node offline with offline message.
  community.general.jenkins_node:
    name: my-node
    state: disabled
    offline_message: >-
      This node is offline for some reason.
"""

RETURN = r"""
url:
  description: URL used to connect to the Jenkins server.
  returned: success
  type: str
  sample: https://jenkins.mydomain.com
user:
  description: User used for authentication.
  returned: success
  type: str
  sample: jenkins
name:
  description: Name of the Jenkins node.
  returned: success
  type: str
  sample: my-node
state:
  description: State of the Jenkins node.
  returned: success
  type: str
  sample: present
created:
  description: Whether or not the Jenkins node was created by the task.
  returned: success
  type: bool
deleted:
  description: Whether or not the Jenkins node was deleted by the task.
  returned: success
  type: bool
disabled:
  description: Whether or not the Jenkins node was disabled by the task.
  returned: success
  type: bool
enabled:
  description: Whether or not the Jenkins node was enabled by the task.
  returned: success
  type: bool
configured:
  description: Whether or not the Jenkins node was configured by the task.
  returned: success
  type: bool
"""

import traceback
from xml.etree import ElementTree as et

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils import _deps as deps

with deps.declare(
    "python-jenkins",
    reason="python-jenkins is required to interact with Jenkins",
    url="https://opendev.org/jjb/python-jenkins",
):
    import jenkins


class JenkinsNode:
    def __init__(self, module: AnsibleModule) -> None:
        self.module = module

        self.name = module.params["name"]
        self.state = module.params["state"]
        self.token = module.params["token"]
        self.user = module.params["user"]
        self.url = module.params["url"]
        self.num_executors = module.params["num_executors"]
        self.labels = module.params["labels"]
        self.offline_message: str | None = module.params["offline_message"]

        if self.offline_message is not None:
            self.offline_message = self.offline_message.strip()

            if self.state != "disabled":
                self.module.fail_json("can not set offline message when state is not disabled")

        if self.labels is not None:
            for label in self.labels:
                if " " in label:
                    self.module.fail_json(f"labels must not contain spaces: got invalid label {label}")

        self.instance = self.get_jenkins_instance()
        self.result = {
            "changed": False,
            "url": self.url,
            "user": self.user,
            "name": self.name,
            "state": self.state,
            "created": False,
            "deleted": False,
            "disabled": False,
            "enabled": False,
            "configured": False,
            "warnings": [],
        }

    def get_jenkins_instance(self):
        try:
            if self.user and self.token:
                return jenkins.Jenkins(self.url, self.user, self.token)
            elif self.user and not self.token:
                return jenkins.Jenkins(self.url, self.user)
            else:
                return jenkins.Jenkins(self.url)
        except Exception as e:
            self.module.fail_json(msg=f"Unable to connect to Jenkins server, {e}")

    def configure_node(self, present):
        if not present:
            # Node would only not be present if in check mode and if not present there
            # is no way to know what would and would not be changed.
            if not self.module.check_mode:
                raise Exception("configure_node present is False outside of check mode")
            return

        configured = False

        data = self.instance.get_node_config(self.name)
        root = et.fromstring(data)

        if self.num_executors is not None:
            elem = root.find("numExecutors")
            if elem is None:
                elem = et.SubElement(root, "numExecutors")
            if elem.text is None or int(elem.text) != self.num_executors:
                elem.text = str(self.num_executors)
                configured = True

        if self.labels is not None:
            elem = root.find("label")
            if elem is None:
                elem = et.SubElement(root, "label")
            labels = []
            if elem.text:
                labels = elem.text.split()
            if labels != self.labels:
                elem.text = " ".join(self.labels)
                configured = True

        if configured:
            data = et.tostring(root, encoding="unicode")

            self.instance.reconfig_node(self.name, data)

        self.result["configured"] = configured
        if configured:
            self.result["changed"] = True

    def present_node(self, configure=True):  # type: (bool) -> bool
        """Assert node present.

        Args:
            configure: If True, run node configuration after asserting node present.

        Returns:
            True if the node is present, False otherwise (i.e. is check mode).
        """

        def create_node():
            try:
                self.instance.create_node(self.name, launcher=jenkins.LAUNCHER_SSH)
            except jenkins.JenkinsException as e:
                # Some versions of python-jenkins < 1.8.3 has an authorization bug when
                # handling redirects returned when posting to resources. If the node is
                # created OK then can ignore the error.
                if not self.instance.node_exists(self.name):
                    self.module.fail_json(msg=f"Create node failed: {e}", exception=traceback.format_exc())

                # TODO: Remove authorization workaround.
                self.result["warnings"].append(
                    "suppressed 401 Not Authorized on redirect after node created: see https://review.opendev.org/c/jjb/python-jenkins/+/931707"
                )

        present = self.instance.node_exists(self.name)
        created = False
        if not present:
            if not self.module.check_mode:
                create_node()
                present = True

            created = True

        if configure:
            self.configure_node(present)

        self.result["created"] = created
        if created:
            self.result["changed"] = True

        return present  # Used to gate downstream queries when in check mode.

    def absent_node(self):
        def delete_node():
            try:
                self.instance.delete_node(self.name)
            except jenkins.JenkinsException as e:
                # Some versions of python-jenkins < 1.8.3 has an authorization bug when
                # handling redirects returned when posting to resources. If the node is
                # deleted OK then can ignore the error.
                if self.instance.node_exists(self.name):
                    self.module.fail_json(msg=f"Delete node failed: {e}", exception=traceback.format_exc())

                # TODO: Remove authorization workaround.
                self.result["warnings"].append(
                    "suppressed 401 Not Authorized on redirect after node deleted: see https://review.opendev.org/c/jjb/python-jenkins/+/931707"
                )

        present = self.instance.node_exists(self.name)
        deleted = False
        if present:
            if not self.module.check_mode:
                delete_node()

            deleted = True

        self.result["deleted"] = deleted
        if deleted:
            self.result["changed"] = True

    def enabled_node(self):
        def get_offline():  # type: () -> bool
            return self.instance.get_node_info(self.name)["offline"]

        present = self.present_node()

        enabled = False

        if present:

            def enable_node():
                try:
                    self.instance.enable_node(self.name)
                except jenkins.JenkinsException as e:
                    # Some versions of python-jenkins < 1.8.3 has an authorization bug when
                    # handling redirects returned when posting to resources. If the node is
                    # disabled OK then can ignore the error.
                    offline = get_offline()

                    if offline:
                        self.module.fail_json(msg=f"Enable node failed: {e}", exception=traceback.format_exc())

                    # TODO: Remove authorization workaround.
                    self.result["warnings"].append(
                        "suppressed 401 Not Authorized on redirect after node enabled: see https://review.opendev.org/c/jjb/python-jenkins/+/931707"
                    )

            offline = get_offline()

            if offline:
                if not self.module.check_mode:
                    enable_node()

                enabled = True
        else:
            # Would have created node with initial state enabled therefore would not have
            # needed to enable therefore not enabled.
            if not self.module.check_mode:
                raise Exception("enabled_node present is False outside of check mode")
            enabled = False

        self.result["enabled"] = enabled
        if enabled:
            self.result["changed"] = True

    def disabled_node(self):
        def get_offline_info():
            info = self.instance.get_node_info(self.name)

            offline = info["offline"]
            offline_message = info["offlineCauseReason"]

            return offline, offline_message

        # Don't configure until after disabled, in case the change in configuration
        # causes the node to pick up a job.
        present = self.present_node(False)

        disabled = False
        changed = False

        if present:
            offline, offline_message = get_offline_info()

            if self.offline_message is not None and self.offline_message != offline_message:
                if offline:
                    # n.b. Internally disable_node uses toggleOffline gated by a not
                    # offline condition. This means that disable_node can not be used to
                    # update an offline message if the node is already offline.
                    #
                    # Toggling the node online to set the message when toggling offline
                    # again is not an option as during this transient online time jobs
                    # may be scheduled on the node which is not acceptable.
                    self.result["warnings"].append("unable to change offline message when already offline")
                else:
                    offline_message = self.offline_message
                    changed = True

            def disable_node():
                try:
                    self.instance.disable_node(self.name, offline_message)
                except jenkins.JenkinsException as e:
                    # Some versions of python-jenkins < 1.8.3 has an authorization bug when
                    # handling redirects returned when posting to resources. If the node is
                    # disabled OK then can ignore the error.
                    offline, _offline_message = get_offline_info()

                    if not offline:
                        self.module.fail_json(msg=f"Disable node failed: {e}", exception=traceback.format_exc())

                    # TODO: Remove authorization workaround.
                    self.result["warnings"].append(
                        "suppressed 401 Not Authorized on redirect after node disabled: see https://review.opendev.org/c/jjb/python-jenkins/+/931707"
                    )

            if not offline:
                if not self.module.check_mode:
                    disable_node()

                disabled = True

        else:
            # Would have created node with initial state enabled therefore would have
            # needed to disable therefore disabled.
            if not self.module.check_mode:
                raise Exception("disabled_node present is False outside of check mode")
            disabled = True

        if disabled:
            changed = True

        self.result["disabled"] = disabled

        if changed:
            self.result["changed"] = True

        self.configure_node(present)


def main():
    module = AnsibleModule(
        argument_spec=dict(
            name=dict(required=True, type="str"),
            url=dict(default="http://localhost:8080"),
            user=dict(),
            token=dict(no_log=True),
            state=dict(choices=["enabled", "disabled", "present", "absent"], default="present"),
            num_executors=dict(type="int"),
            labels=dict(type="list", elements="str"),
            offline_message=dict(type="str"),
        ),
        supports_check_mode=True,
    )

    deps.validate(module)

    jenkins_node = JenkinsNode(module)

    state = module.params.get("state")
    if state == "enabled":
        jenkins_node.enabled_node()
    elif state == "disabled":
        jenkins_node.disabled_node()
    elif state == "present":
        jenkins_node.present_node()
    else:
        jenkins_node.absent_node()

    module.exit_json(**jenkins_node.result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_switch_routing_ospf
short_description: Resource module for networks _switch _routing _ospf
description:
  - Manage operation update of the resource networks _switch _routing _ospf.
  - Update layer 3 OSPF routing configuration.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  areas:
    description: OSPF areas.
    elements: dict
    suboptions:
      areaId:
        description: OSPF area ID.
        type: str
      areaName:
        description: Name of the OSPF area.
        type: str
      areaType:
        description: Area types in OSPF. Must be one of "normal", "stub", "nssa".
        type: str
    type: list
  deadTimerInSeconds:
    description: Time interval to determine when the peer will be declared inactive/dead.
      Value must be between 1 and 65535.
    type: int
  enabled:
    description: Boolean value to enable or disable OSPF routing. OSPF routing is
      disabled by default.
    type: bool
  helloTimerInSeconds:
    description: Time interval in seconds at which hello packet will be sent to OSPF
      neighbors to maintain connectivity. Value must be between 1 and 255. Default
      is 10 seconds.
    type: int
  md5AuthenticationEnabled:
    description: Boolean value to enable or disable MD5 authentication. MD5 authentication
      is disabled by default.
    type: bool
  md5AuthenticationKey:
    description: MD5 authentication credentials. This param is only relevant if md5AuthenticationEnabled
      is true.
    suboptions:
      id:
        description: MD5 authentication key index. Key index must be between 1 to
          255.
        type: int
      passphrase:
        description: MD5 authentication passphrase.
        type: str
    type: dict
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  v3:
    description: OSPF v3 configuration.
    suboptions:
      areas:
        description: OSPF v3 areas.
        elements: dict
        suboptions:
          areaId:
            description: OSPF area ID.
            type: str
          areaName:
            description: Name of the OSPF area.
            type: str
          areaType:
            description: Area types in OSPF. Must be one of "normal", "stub", "nssa".
            type: str
        type: list
      deadTimerInSeconds:
        description: Time interval to determine when the peer will be declared inactive/dead.
          Value must be between 1 and 65535.
        type: int
      enabled:
        description: Boolean value to enable or disable V3 OSPF routing. OSPF V3 routing
          is disabled by default.
        type: bool
      helloTimerInSeconds:
        description: Time interval in seconds at which hello packet will be sent to
          OSPF neighbors to maintain connectivity. Value must be between 1 and 255.
          Default is 10 seconds.
        type: int
    type: dict
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for switch updateNetworkSwitchRoutingOspf
    description: Complete reference of the updateNetworkSwitchRoutingOspf API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-network-switch-routing-ospf
notes:
  - SDK Method used are
    switch.Switch.update_network_switch_routing_ospf,
  - Paths used are
    put /networks/{networkId}/switch/routing/ospf,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.networks_switch_routing_ospf:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    areas:
      - areaId: '1284392014819'
        areaName: Backbone
        areaType: normal
    deadTimerInSeconds: 40
    enabled: true
    helloTimerInSeconds: 10
    md5AuthenticationEnabled: true
    md5AuthenticationKey:
      id: 1234
      passphrase: abc1234
    networkId: string
    v3:
      areas:
        - areaId: '1284392014819'
          areaName: V3 Backbone
          areaType: normal
      deadTimerInSeconds: 40
      enabled: true
      helloTimerInSeconds: 10
    vrf: Blue
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "enabled": true,
      "helloTimerInSeconds": 0,
      "deadTimerInSeconds": 0,
      "areas": [
        {
          "areaId": "string",
          "areaName": "string",
          "areaType": "string"
        }
      ],
      "v3": {
        "enabled": true,
        "helloTimerInSeconds": 0,
        "deadTimerInSeconds": 0,
        "areas": [
          {
            "areaId": "string",
            "areaName": "string",
            "areaType": "string"
          }
        ]
      },
      "md5AuthenticationEnabled": true,
      "md5AuthenticationKey": {
        "id": 0,
        "passphrase": "string"
      },
      "vrf": {
        "name": "string"
      }
    }
"""



#!/usr/bin/python
# Copyright (c) 2025, Alexei Znamensky <russoz@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: django_loaddata
author:
  - Alexei Znamensky (@russoz)
short_description: Wrapper for C(django-admin loaddata)
version_added: 11.3.0
description:
  - This module is a wrapper for the execution of C(django-admin loaddata).
extends_documentation_fragment:
  - community.general._attributes
  - community.general._django
  - community.general._django.database
  - community.general._django.data
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
options:
  app:
    description: Specifies a single app to look for fixtures in rather than looking in all apps.
    type: str
  ignore_non_existent:
    description: Ignores fields and models that may have been removed since the fixture was originally generated.
    type: bool
  fixtures:
    description:
      - List of paths to the fixture files.
    type: list
    elements: path
"""

EXAMPLES = r"""
- name: Dump all data
  community.general.django_dumpdata:
    settings: myproject.settings

- name: Create cache table in the other database
  community.general.django_createcachetable:
    database: myotherdb
    settings: fancysite.settings
    pythonpath: /home/joedoe/project/fancysite
    venv: /home/joedoe/project/fancysite/venv
"""

RETURN = r"""
run_info:
  description: Command-line execution information.
  type: dict
  returned: success and O(verbosity) >= 3
version:
  description: Version of Django.
  type: str
  returned: always
  sample: 5.1.2
"""

from ansible_collections.community.general.plugins.module_utils._django import DjangoModuleHelper


class DjangoLoadData(DjangoModuleHelper):
    module = dict(
        argument_spec=dict(
            app=dict(type="str"),
            ignore_non_existent=dict(type="bool"),
            fixtures=dict(type="list", elements="path"),
        ),
        supports_check_mode=False,
    )
    django_admin_cmd = "loaddata"
    django_admin_arg_order = "database_dash ignore_non_existent app format excludes fixtures"
    _django_args = ["data", "database_dash"]

    def __init_module__(self):
        self.vars.set("database_dash", self.vars.database, output=False)


def main():
    DjangoLoadData.execute()


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
# Python SDK Reference: https://learn.microsoft.com/en-us/python/api/azure-mgmt-cdn/azure.mgmt.cdn.operations.afdoriginsoperations?view=azure-python
from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_afdorigin
version_added: "3.4.0"
short_description: Manage an Azure Front Door Origin to be used with Standard or Premium Frontdoor.
description:
    - Create, update and delete an Azure Front Door (AFD) Origin to be used by a Front Door Service Profile created using azure_rm_cdnprofile.

options:
    azure_origin:
        description:
            - Resource reference to the AFD origin resource.
        type: str
    enabled_state:
        description:
            - Whether to enable health probes to be made against backends defined under backend pools.
            - Health probes can only be disabled if there is a single enabled backend in single enabled backend pool.
        type: str
        choices:
            - Enabled
            - Disabled
    host_name:
        description:
            - The address of the origin. Domain names, IPv4 addresses, and IPv6 addresses are supported.
            - This should be unique across all origins in an endpoint.
        type: str
    http_port:
        description:
            - The value of the HTTP port. Must be between 1 and 65535.
        default: 80
        type: int
    https_port:
        description:
            - The value of the HTTPS port. Must be between 1 and 65535.
        default: 443
        type: int
    name:
        description:
            - Name of the origin that is unique within the AFD Profile.
        required: true
        type: str
    origin_host_header:
        description:
            - The host header value sent to the origin with each request. If you leave this blank, the request hostname determines this value.
            - Azure Front Door origins, such as Web Apps, Blob Storage,
            - and Cloud Services require this host header value to match the origin hostname by default.
            - This overrides the host header defined at the AFD Endpoint.
        type: str
    origin_group_name:
        description:
            - Name of the origin group which is unique within the profile.
        required: true
        type: str
    priority:
        description:
            - Priority of origin in given origin group for load balancing.
            - Higher priorities will not be used for load balancing if any lower priority origin is healthy. Must be between 1 and 5.
        type: int
    profile_name:
        description:
            - Name of the AFD Profile.
        required: true
        type: str
    resource_group:
        description:
            - Name of a resource group where the AFD Origin exists or will be created.
        required: true
        type: str
    shared_private_link_resource:
        description:
            - The properties of the private link resource for private origin.
        type: dict
        suboptions:
            group_id:
                description:
                    - The group id from the provider of resource the shared private link resource is for.
                type: str
            private_link:
                description:
                    - The resource id of the resource the shared private link resource is for.
                type: str
            private_link_location:
                description:
                    - The location of the shared private link resource.
                type: str
            request_message:
                description:
                    - The request message for requesting approval of the shared private link resource.
                type: str
            status:
                description:
                    - Status of the shared private link resource. Can be Pending, Approved, Rejected, Disconnected, or Timeout.
                type: str
                default: Approved
                choices:
                    - Approved
                    - Disconnected
                    - Pending
                    - Rejected
                    - Timeout
    state:
        description:
            - Assert the state of the AFD Profile. Use C(present) to create or update an AFD profile and C(absent) to delete it.
        default: present
        type: str
        choices:
            - absent
            - present
    weight:
        description:
            - Weight of the origin in given origin group for load balancing. Must be between 1 and 1000.
        type: int

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Jarret Tooley (@jartoo)
'''

EXAMPLES = '''
- name: Create an AFD Origin
  azure_rm_afdorigin:
    name: myOrigin
    origin_group_name: myOriginGroup
    profile_name: myProfile
    resource_group: myResourceGroup
    state: present
    host_name: "10.0.0.1"
    origin_host_header: "10.0.0.1"
    http_port: 80
    https_port: 443
    priority: 1
    weight: 1000

- name: Delete an AFD Origin
  azure_rm_afdorigin:
    name: myOrigin
    origin_group_name: myOriginGroup
    profile_name: myProfile
    resource_group: myResourceGroup
    state: absent
'''
RETURN = '''
id:
    description:
        - ID of the AFD Origin.
    returned: always
    type: str
    sample: "id: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxx/resourcegroups/myRG/providers/Microsoft.Cdn/profiles/myProf/origingroups/myOG/origins/myO"
host_name:
    description:
        - Host name of the AFD Origin.
    returned: always
    type: str
    sample: "myorigin.azurefd.net"

'''
import re
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase


try:
    from azure.mgmt.cdn.models import AFDOrigin, AFDOriginUpdateParameters, SharedPrivateLinkResourceProperties, ResourceReference
except ImportError as ec:
    # This is handled in azure_rm_common
    pass


def origin_to_dict(origin):
    ''' Convert object to dict '''
    return dict(
        azure_origin=origin.azure_origin,
        deployment_status=origin.deployment_status,
        enabled_state=origin.enabled_state,
        # enforce_certificate_check = origin.enforce_certificate_check, # Not fully implemented yet
        host_name=origin.host_name,
        http_port=origin.http_port,
        https_port=origin.https_port,
        id=origin.id,
        name=origin.name,
        origin_group_name=re.sub('\\/.*', '', re.sub('.*origingroups\\/', '', origin.id)),
        origin_host_header=origin.origin_host_header,
        priority=origin.priority,
        provisioning_state=origin.provisioning_state,
        shared_private_link_resource=origin.shared_private_link_resource,
        type=origin.type,
        weight=origin.weight
    )


class AzureRMOrigin(AzureRMModuleBase):
    ''' Class for managing Origin '''
    def __init__(self):
        self.module_arg_spec = dict(
            azure_origin=dict(type='str'),
            enabled_state=dict(type='str', choices=['Enabled', 'Disabled']),
            # enforce_certification_name_check=dict(type='bool'),
            host_name=dict(type='str'),
            http_port=dict(type='int', default=80),
            https_port=dict(type='int', default=443),
            name=dict(
                type='str',
                required=True
            ),
            origin_group_name=dict(
                type='str',
                required=True
            ),
            origin_host_header=dict(type='str'),
            priority=dict(type='int'),
            profile_name=dict(
                type='str',
                required=True
            ),
            resource_group=dict(
                type='str',
                required=True
            ),
            shared_private_link_resource=dict(
                type='dict',
                options=dict(
                    group_id=dict(type='str'),
                    private_link=dict(type='str'),
                    private_link_location=dict(type='str'),
                    request_message=dict(type='str'),
                    status=dict(
                        type='str',
                        default='Approved',
                        choices=[
                            "Pending",
                            "Approved",
                            "Rejected",
                            "Disconnected",
                            "Timeout"
                        ]
                    )
                )
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            ),
            weight=dict(type='int')
        )

        self.azure_origin = None
        self.enabled_state = None
        self.host_name = None
        self.http_port = None
        self.https_port = None
        self.origin_host_header = None
        self.priority = None
        self.shared_private_link_resource = None
        self.weight = None

        self.name = None
        self.origin_group_name = None
        self.profile_name = None
        self.resource_group = None
        self.state = None

        self.results = dict(changed=False)

        super(AzureRMOrigin, self).__init__(
            derived_arg_spec=self.module_arg_spec,
            supports_check_mode=True,
            supports_tags=False)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        to_be_updated = False

        response = self.get_origin()

        if self.state == 'present':

            if not response:
                self.log("Need to create the Origin")

                if not self.check_mode:
                    new_response = self.create_origin()
                    self.results['id'] = new_response['id']
                    self.results['host_name'] = new_response['host_name']

                self.results['changed'] = True

            else:
                self.log('Results : {0}'.format(response))
                self.results['id'] = response['id']
                self.results['host_name'] = response['host_name']

                if response['host_name'] != self.host_name and self.host_name:
                    to_be_updated = True
                if response['http_port'] != self.http_port and self.http_port:
                    to_be_updated = True
                if response['https_port'] != self.https_port and self.https_port:
                    to_be_updated = True
                if response['origin_host_header'] != self.origin_host_header and self.origin_host_header:
                    to_be_updated = True
                if response['priority'] != self.priority and self.priority:
                    to_be_updated = True
                if response['weight'] != self.weight and self.weight:
                    to_be_updated = True
                if response['enabled_state'] != self.enabled_state and self.enabled_state:
                    to_be_updated = True
                if response['shared_private_link_resource'] != self.shared_private_link_resource and self.shared_private_link_resource:
                    to_be_updated = True
                elif response['shared_private_link_resource']:
                    if response['shared_private_link_resource']['group_id'] != \
                            self.shared_private_link_resource.get('group_id') and \
                            self.shared_private_link_resource.get('group_id'):
                        to_be_updated = True
                    if response['shared_private_link_resource']['private_link'] != \
                            self.shared_private_link_resource.get('private_link') and \
                            self.shared_private_link_resource.get('private_link'):
                        to_be_updated = True
                    if response['shared_private_link_resource']['private_link_location'] != \
                            self.shared_private_link_resource.get('private_link_location') and \
                            self.shared_private_link_resource.get('private_link_location'):
                        to_be_updated = True
                    if response['shared_private_link_resource']['request_message'] != self.origin['shared_private_link_resource']['request_message'] and \
                            self.shared_private_link_resource.get('request_message'):
                        to_be_updated = True
                    if response['shared_private_link_resource']['status'] != self.shared_private_link_resource.get('status') and \
                            self.shared_private_link_resource.get('status'):
                        to_be_updated = True

                if to_be_updated:
                    self.log("Need to update the Origin")

                    if not self.check_mode:
                        new_response = self.update_origin()
                        self.results['id'] = new_response['id']
                        self.results['host_name'] = new_response['host_name']

                    self.results['changed'] = True

        elif self.state == 'absent':
            if not response:
                self.log("Origin {0} does not exist.".format(self.name))
                self.results['id'] = ""
                self.results['host_name'] = ""
            else:
                self.log("Need to delete the Origin")
                self.results['changed'] = True
                self.results['id'] = response['id']
                self.results['host_name'] = response['host_name']

                if not self.check_mode:
                    self.delete_origin()

        return self.results

    def create_origin(self):
        '''
        Creates a Azure Origin.

        :return: deserialized Azure Origin instance state dictionary
        '''
        self.log("Creating the Azure Origin instance {0}".format(self.name))

        shared_private_link_resource = None
        if self.shared_private_link_resource:
            shared_private_link_resource = SharedPrivateLinkResourceProperties(
                group_id=self.shared_private_link_resource.get('group_id'),
                private_link=ResourceReference(id=self.shared_private_link_resource.get('private_link')),
                private_link_location=self.shared_private_link_resource.get('private_link_location'),
                request_message=self.shared_private_link_resource.get('request_message'),
                status=self.shared_private_link_resource.get('status')
            )

        parameters = AFDOrigin(
            azure_origin=self.azure_origin,
            host_name=self.host_name,
            http_port=self.http_port,
            https_port=self.https_port,
            origin_host_header=self.origin_host_header,
            priority=self.priority,
            weight=self.weight,
            enabled_state=self.enabled_state,
            shared_private_link_resource=shared_private_link_resource
        )

        try:
            poller = self.cdn_client.afd_origins.begin_create(resource_group_name=self.resource_group,
                                                              profile_name=self.profile_name,
                                                              origin_group_name=self.origin_group_name,
                                                              origin_name=self.name,
                                                              origin=parameters)
            response = self.get_poller_result(poller)
            return origin_to_dict(response)
        except Exception as exc:
            self.log('Error attempting to create Azure Origin instance.')
            self.fail("Error Creating Azure Origin instance: {0}".format(str(exc)))

    def update_origin(self):
        '''
        Updates an Azure Origin.

        :return: deserialized Azure Origin instance state dictionary
        '''
        self.log("Updating the Azure Origin instance {0}".format(self.name))

        shared_private_link_resource = None
        if self.shared_private_link_resource:
            shared_private_link_resource = SharedPrivateLinkResourceProperties(
                group_id=self.shared_private_link_resource.get('group_id'),
                private_link=ResourceReference(id=self.shared_private_link_resource.get('private_link')),
                private_link_location=self.shared_private_link_resource.get('private_link_location'),
                request_message=self.shared_private_link_resource.get('request_message'),
                status=self.shared_private_link_resource.get('status')
            )

        parameters = AFDOriginUpdateParameters(
            azure_origin=self.azure_origin,
            host_name=self.host_name,
            http_port=self.http_port,
            https_port=self.https_port,
            origin_host_header=self.origin_host_header,
            priority=self.priority,
            weight=self.weight,
            enabled_state=self.enabled_state,
            shared_private_link_resource=shared_private_link_resource
        )

        try:
            poller = self.cdn_client.afd_origins.begin_update(resource_group_name=self.resource_group,
                                                              profile_name=self.profile_name,
                                                              origin_group_name=self.origin_group_name,
                                                              origin_name=self.name,
                                                              origin_update_properties=parameters)
            response = self.get_poller_result(poller)
            return origin_to_dict(response)
        except Exception as exc:
            self.log('Error attempting to update Azure Origin instance.')
            self.fail("Error updating Azure Origin instance: {0}".format(str(exc)))

    def delete_origin(self):
        '''
        Deletes the specified Azure Origin in the specified subscription and resource group.

        :return: True
        '''
        self.log("Deleting the Origin {0}".format(self.name))
        try:
            poller = self.cdn_client.afd_origins.begin_delete(resource_group_name=self.resource_group,
                                                              profile_name=self.profile_name,
                                                              origin_group_name=self.origin_group_name,
                                                              origin_name=self.name)
            self.get_poller_result(poller)
            return True
        except Exception as exc:
            self.log('Error attempting to delete the Origin.')
            self.fail("Error deleting the Origin: {0}".format(str(exc)))
            return False

    def get_origin(self):
        '''
        Gets the properties of the specified Origin.

        :return: deserialized Origin state dictionary
        '''
        self.log(
            "Checking if the Origin {0} is present".format(self.name))
        try:
            response = self.cdn_client.afd_origins.get(resource_group_name=self.resource_group,
                                                       profile_name=self.profile_name,
                                                       origin_group_name=self.origin_group_name,
                                                       origin_name=self.name)
            self.log("Response : {0}".format(response))
            self.log("Origin : {0} found".format(response.name))
            return origin_to_dict(response)
        except Exception as exc:
            self.log('Did not find the Origin. {0}'.format(str(exc)))
            return False


def main():
    """Main execution"""
    AzureRMOrigin()


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later
# SPDX-FileCopyrightText: 2026 Christoph Fiehe <christoph.fiehe@gmail.com>

from __future__ import annotations

DOCUMENTATION = r"""
module: icinga2_downtime
short_description: Manages Icinga 2 downtimes
version_added: "12.4.0"
description:
  - Manages downtimes in Icinga 2 through its REST API.
  - Options as described at U(https://icinga.com/docs/icinga-2/latest/doc/12-icinga2-api/#schedule-downtime).
author:
  - Christoph Fiehe (@cfiehe)
attributes:
  check_mode:
    support: none
    details:
      - In case of a complex filter expression, it may become very complex to decide
        whether downtime creation or removal will succeed and trigger a change.
  diff_mode:
    support: none
options:
  all_services:
    description:
      - Whether downtimes should be created for all services of the matched host objects.
      - If omitted, Icinga 2 does not create downtimes for all services of the matched host objects by default.
    type: bool
  author:
    description:
      - Name of the author.
    type: str
    default: "Ansible"
  comment:
    description:
      - A descriptive comment.
    type: str
    default: Downtime scheduled by Ansible
  child_options:
    description:
      - Schedule child downtimes.
    type: str
    choices: ["DowntimeNoChildren", "DowntimeTriggeredChildren", "DowntimeNonTriggeredChildren"]
  duration:
    description:
      - Duration of the downtime.
      - Required in case of a flexible downtime.
    type: int
  end_time:
    description:
      - End time of the downtime as UNIX timestamp.
    type: int
  filter_vars:
    description:
      - Variable names and values used in the filter expression.
    type: dict
  filter:
    description:
      - Filter expression limiting the objects to operate on.
    type: str
  fixed:
    description:
      - Whether the downtime is fixed or flexible.
      - If omitted, Icinga 2 creates a fixed downtime by default.
    type: bool
  name:
    description:
      - Name of the downtime object.
      - This option has no effect for states other than V(absent).
    type: str
  object_type:
    description:
      - Use V(Host) for a host downtime and V(Service) for a service downtime.
      - Use V(Downtime) and give the name of the downtime object you want to remove.
    type: str
    choices: ["Service", "Host", "Downtime"]
    default: Host
  start_time:
    description:
      - Start time of the downtime as UNIX timestamp.
    type: int
  state:
    description:
      - State of the downtime.
    type: str
    choices: ["present", "absent"]
    default: present
  trigger_name:
    description:
      - Name of the downtime trigger.
    type: str
extends_documentation_fragment:
  - community.general._icinga2_api
  - community.general._attributes
  - ansible.builtin.url
"""

EXAMPLES = r"""
- name: Schedule a host downtime
  community.general.icinga2_downtime:
    url: "https://icinga2.example.com:5665"
    url_username: icingadmin
    url_password: secret
    state: present
    author: Ansible
    comment: Scheduled downtime for test purposes.
    all_services: true
    start_time: "{{ downtime_start_time }}"
    end_time: "{{ downtime_end_time }}"
    duration: "{{ downtime_duration }}"
    fixed: true
    object_type: Host
    filter: host.name=="host.example.com"
  delegate_to: localhost
  register: icinga2_downtime_response
  vars:
    downtime_start_time: "{{ ansible_date_time['epoch'] | int }}"
    downtime_end_time: "{{ downtime_start_time | int + 3600 }}"
    downtime_duration: "{{ downtime_end_time | int - downtime_start_time | int }}"

- name: Remove scheduled host downtime
  community.general.icinga2_downtime:
    url: "https://icinga2.example.com:5665"
    url_username: icingadmin
    url_password: secret
    state: absent
    author: Ansible
    object_type: Downtime
    name: "{{ icinga2_downtime_response.results[0].name }}"
  delegate_to: localhost
  when: icinga2_downtime_response.results | default([]) | length > 0
"""

RETURN = r"""
# Returns the results of downtime scheduling as a list of JSON dictionaries from the Icinga 2 API under the C(results) key.
# Refer to https://icinga.com/docs/icinga-2/latest/doc/12-icinga2-api/#schedule-downtime for more details.
results:
  description: Results of downtime scheduling or removal
  type: list
  returned: success
  elements: dict
  contains:
    code:
      description: Success or error code of downtime scheduling.
      returned: always
      type: int
      sample: 200
    legacy_id:
      description: Legacy id of the downtime object.
      returned: if a downtime was scheduled successfully
      type: int
      sample: 28911
    name:
      description: Name of the downtime object.
      returned: if a downtime was scheduled successfully
      type: str
      sample: host.example.com!e19c705a-54c2-49c5-8014-70ff624f9e51
    status:
      description: Human-readable message describing the result of downtime scheduling.
      returned: always
      type: str
      sample: Successfully scheduled downtime 'host.example.com!e19c705a-54c2-49c5-8014-70ff624f9e51' for object 'host.example.com'.
  sample:
    [
      {
        "code": 200,
        "legacy_id": 28911,
        "name": "host.example.com!e19c705a-54c2-49c5-8014-70ff624f9e51",
        "status": "Successfully scheduled downtime 'host.example.com!e19c705a-54c2-49c5-8014-70ff624f9e51' for object 'host.example.com'.",
      }
    ]
error:
  description: Error message as JSON dictionary returned from the Icinga 2 API.
  type: dict
  returned: if downtime scheduling or removal did not succeed
  sample:
    {
      "error": 404,
      "status": "No objects found."
    }
"""

import json
from contextlib import suppress

from ansible_collections.community.general.plugins.module_utils._icinga2 import (
    Icinga2Client,
    icinga2_argument_spec,
)
from ansible_collections.community.general.plugins.module_utils._module_helper import StateModuleHelper


class Icinga2Downtime(StateModuleHelper):
    argument_spec = icinga2_argument_spec()
    argument_spec.update(
        all_services=dict(type="bool"),
        author=dict(type="str", default="Ansible"),
        comment=dict(type="str", default="Downtime scheduled by Ansible"),
        child_options=dict(
            type="str",
            choices=[
                "DowntimeNoChildren",
                "DowntimeTriggeredChildren",
                "DowntimeNonTriggeredChildren",
            ],
        ),
        duration=dict(type="int"),
        end_time=dict(type="int"),
        filter_vars=dict(type="dict"),
        filter=dict(type="str"),
        fixed=dict(type="bool"),
        name=dict(type="str"),
        object_type=dict(type="str", choices=["Service", "Host", "Downtime"], default="Host"),
        start_time=dict(type="int"),
        state=dict(type="str", choices=["present", "absent"], default="present"),
        trigger_name=dict(type="str"),
    )
    module = dict(
        argument_spec=argument_spec,
        supports_check_mode=False,
        required_if=(
            (
                "state",
                "present",
                ["comment", "start_time", "end_time", "filter"],
            ),
            ("fixed", False, ["duration"]),
        ),
        required_one_of=[["filter", "name"]],
    )

    def __init_module__(self) -> None:
        self.client = Icinga2Client(
            module=self.module,  # type:ignore[arg-type]
            url=self.vars.url,
            ca_path=self.vars.ca_path,
            timeout=self.vars.timeout,
        )

    def state_present(self) -> None:
        duration = self.vars.duration
        end_time = self.vars.end_time
        start_time = self.vars.start_time

        if end_time <= start_time:
            self.do_raise(msg="The end time must be later than the start time.")

        if duration is None:
            duration = end_time - start_time

        response, info = self.client.actions.schedule_downtime(
            all_services=self.vars.all_services,
            author=self.vars.author,
            child_options=self.vars.child_options,
            comment=self.vars.comment,
            duration=duration,
            end_time=end_time,
            filter_vars=self.vars.filter_vars,
            filter=self.vars.filter,
            fixed=self.vars.fixed,
            object_type=self.vars.object_type,
            start_time=start_time,
            trigger_name=self.vars.trigger_name,
        )

        status_code = info["status"]

        if 200 <= status_code <= 299:
            self.vars.set("results", json.loads(response.read())["results"], output=True)
            self.vars.msg = "Successfully scheduled downtime."
            self.changed = True
        elif status_code >= 400:
            with suppress(KeyError, ValueError):
                self.vars.set("error", json.loads(info["body"]))  # type:ignore[arg-type]

            self.do_raise(msg="Unable to schedule downtime.")

    def state_absent(self) -> None:
        response, info = self.client.actions.remove_downtime(
            filter_vars=self.vars.filter_vars,
            filter=self.vars.filter,
            name=self.vars.name,
            object_type=self.vars.object_type,
        )

        status_code = info["status"]

        if 200 <= status_code <= 299:
            self.vars.set("results", json.loads(response.read())["results"], output=True)
            self.vars.msg = "Successfully removed downtime."
            self.changed = True
        elif status_code == 404:
            self.vars.msg = "No matching downtime object found."
        elif status_code >= 400:
            with suppress(KeyError, ValueError):
                self.vars.set("error", json.loads(info["body"]))  # type:ignore[arg-type]

            self.do_raise(msg="Unable to remove downtime.")


def main():
    Icinga2Downtime.execute()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Copyright 2025 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the
# ansible.content_builder.
#
# Manually editing this file is not advised.
#
# To update the argspec make the desired changes
# in the documentation in the module file and re-run
# ansible.content_builder commenting out
# the path to external 'docstring' in build.yaml.
#
##############################################

"""
The arg spec for the ios_hsrp_interfaces module
"""


class Hsrp_interfacesArgs(object):  # pylint: disable=R0903
    """The arg spec for the ios_hsrp_interfaces module"""

    argument_spec = {
        "config": {
            "type": "list",
            "elements": "dict",
            "options": {
                "name": {"type": "str", "required": True},
                "bfd": {"type": "bool"},
                "delay": {
                    "type": "dict",
                    "options": {
                        "minimum": {"type": "int"},
                        "reload": {"type": "int"},
                    },
                },
                "redirect": {
                    "type": "dict",
                    "options": {
                        "advertisement": {
                            "type": "dict",
                            "options": {
                                "authentication": {
                                    "type": "dict",
                                    "no_log": False,
                                    "options": {
                                        "key_chain": {
                                            "type": "str",
                                            "no_log": False,
                                        },
                                        "key_string": {
                                            "type": "str",
                                            "no_log": True,
                                        },
                                        "encryption": {
                                            "type": "str",
                                            "no_log": False,
                                        },
                                        "time_out": {"type": "int"},
                                        "password_text": {
                                            "type": "str",
                                            "no_log": True,
                                        },
                                    },
                                },
                            },
                        },
                        "timers": {
                            "type": "dict",
                            "options": {
                                "adv_timer": {"type": "int"},
                                "holddown_timer": {"type": "int"},
                            },
                        },
                    },
                },
                "mac_refresh": {"type": "int"},
                "use_bia": {
                    "type": "dict",
                    "options": {
                        "scope": {"type": "bool"},
                        "set": {"type": "bool"},
                    },
                },
                "version": {"type": "int", "choices": [1, 2]},
                "standby_options": {
                    "type": "list",
                    "elements": "dict",
                    "aliases": ["standby_groups"],
                    "options": {
                        "group_no": {"type": "int"},
                        "follow": {"type": "str"},
                        "ip": {
                            "type": "list",
                            "elements": "dict",
                            "options": {
                                "virtual_ip": {"type": "str"},
                                "secondary": {"type": "bool"},
                            },
                        },
                        "ipv6": {
                            "type": "dict",
                            "options": {
                                "addresses": {
                                    "type": "list",
                                    "elements": "str",
                                },
                                "autoconfig": {"type": "bool"},
                            },
                        },
                        "mac_address": {"type": "str"},
                        "group_name": {"type": "str"},
                        "authentication": {
                            "type": "dict",
                            "options": {
                                "key_chain": {
                                    "type": "str",
                                    "no_log": False,
                                },
                                "key_string": {
                                    "type": "str",
                                    "no_log": False,
                                },
                                "encryption": {"type": "int"},
                                "time_out": {"type": "int"},
                                "password_text": {
                                    "type": "str",
                                    "no_log": True,
                                },
                            },
                        },
                        "preempt": {
                            "type": "dict",
                            "options": {
                                "enabled": {"type": "bool"},
                                "minimum": {"type": "int"},
                                "reload": {"type": "int"},
                                "sync": {"type": "int"},
                                "delay": {"type": "bool"},
                            },
                        },
                        "priority": {"type": "int", "default": 100},
                        "timers": {
                            "type": "dict",
                            "options": {
                                "hello_interval": {"type": "int"},
                                "hold_time": {"type": "int"},
                                "msec": {
                                    "type": "dict",
                                    "options": {
                                        "hello_interval": {"type": "int"},
                                        "hold_time": {"type": "int"},
                                    },
                                },
                            },
                        },
                        "track": {
                            "type": "list",
                            "elements": "dict",
                            "options": {
                                "track_no": {"type": "int"},
                                "decrement": {"type": "int"},
                                "shutdown": {"type": "bool"},
                            },
                        },
                    },
                },
            },
        },
        "running_config": {"type": "str"},
        "state": {
            "choices": [
                "merged",
                "replaced",
                "overridden",
                "deleted",
                "rendered",
                "gathered",
                "parsed",
            ],
            "default": "merged",
            "type": "str",
        },
    }  # pylint: disable=C0301



# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible_collections.community.aws.plugins.module_utils.base import BaseWaiterFactory


class DynamodbWaiterFactory(BaseWaiterFactory):
    def __init__(self, module):
        # the AWSRetry wrapper doesn't support the wait functions (there's no
        # public call we can cleanly wrap)
        client = module.client("dynamodb")
        super().__init__(module, client)

    @property
    def _waiter_model_data(self):
        data = super()._waiter_model_data
        ddb_data = dict(
            table_exists=dict(
                operation="DescribeTable",
                delay=20,
                maxAttempts=25,
                acceptors=[
                    dict(expected="ACTIVE", matcher="path", state="success", argument="Table.TableStatus"),
                    dict(expected="ResourceNotFoundException", matcher="error", state="retry"),
                ],
            ),
            table_not_exists=dict(
                operation="DescribeTable",
                delay=20,
                maxAttempts=25,
                acceptors=[
                    dict(expected="ResourceNotFoundException", matcher="error", state="success"),
                ],
            ),
            global_indexes_active=dict(
                operation="DescribeTable",
                delay=20,
                maxAttempts=25,
                acceptors=[
                    dict(expected="ResourceNotFoundException", matcher="error", state="failure"),
                    # If there are no secondary indexes, simply return
                    dict(
                        expected=False,
                        matcher="path",
                        state="success",
                        argument="contains(keys(Table), `GlobalSecondaryIndexes`)",
                    ),
                    dict(
                        expected="ACTIVE",
                        matcher="pathAll",
                        state="success",
                        argument="Table.GlobalSecondaryIndexes[].IndexStatus",
                    ),
                    dict(
                        expected="CREATING",
                        matcher="pathAny",
                        state="retry",
                        argument="Table.GlobalSecondaryIndexes[].IndexStatus",
                    ),
                    dict(
                        expected="UPDATING",
                        matcher="pathAny",
                        state="retry",
                        argument="Table.GlobalSecondaryIndexes[].IndexStatus",
                    ),
                    dict(
                        expected="DELETING",
                        matcher="pathAny",
                        state="retry",
                        argument="Table.GlobalSecondaryIndexes[].IndexStatus",
                    ),
                    dict(
                        expected=True,
                        matcher="path",
                        state="success",
                        argument="length(Table.GlobalSecondaryIndexes) == `0`",
                    ),
                ],
            ),
        )
        data.update(ddb_data)
        return data


def _do_wait(module, waiter_name, action_description, wait_timeout, table_name):
    delay = min(wait_timeout, 5)
    max_attempts = wait_timeout // delay

    try:
        waiter = DynamodbWaiterFactory(module).get_waiter(waiter_name)
        waiter.wait(
            WaiterConfig={"Delay": delay, "MaxAttempts": max_attempts},
            TableName=table_name,
        )
    except botocore.exceptions.WaiterError as e:
        module.fail_json_aws(e, msg=f"Timeout while waiting for {action_description}")
    except (
        botocore.exceptions.ClientError,
        botocore.exceptions.BotoCoreError,
    ) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e, msg=f"Failed while waiting for {action_description}")


def wait_table_exists(module, wait_timeout, table_name):
    _do_wait(module, "table_exists", "table creation", wait_timeout, table_name)


def wait_table_not_exists(module, wait_timeout, table_name):
    _do_wait(module, "table_not_exists", "table deletion", wait_timeout, table_name)


def wait_indexes_active(module, wait_timeout, table_name):
    _do_wait(module, "global_indexes_active", "secondary index updates", wait_timeout, table_name)



# -*- coding: utf-8 -*-
# Copyright 2023 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The Evpn_evi parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


class Evpn_eviTemplate(NetworkTemplate):
    def __init__(self, lines=None, module=None):
        super(Evpn_eviTemplate, self).__init__(lines=lines, tmplt=self, module=module)

    # fmt: off
    PARSERS = [
        {
            "name": "evi",
            "getval": re.compile(
                r"""^l2vpn\sevpn\sinstance\s
                    (?P<evi>\d+)\svlan-based
                $""",
                re.VERBOSE,
            ),
            "setval": "l2vpn evpn instance {{ evi }} vlan-based",
            "result": {"{{ evi }}": {"evi": "{{ evi }}"}},
            "shared": True,
        },
        {
            "name": "default_gateway.advertise.enable",
            "getval": re.compile(
                r"""
                \s+default-gateway\sadvertise\senable
                $""",
                re.VERBOSE,
            ),
            "setval": "default-gateway advertise enable",
            "result": {"{{ evi }}": {"default_gateway": {"advertise": {"enable": True}}}},
        },
        {
            "name": "default_gateway.advertise.disable",
            "getval": re.compile(
                r"""
                \s+default-gateway\sadvertise\sdisable
                $""",
                re.VERBOSE,
            ),
            "setval": "default-gateway advertise disable",
            "result": {"{{ evi }}": {"default_gateway": {"advertise": {"disable": True}}}},
        },
        {
            "name": "encapsulation",
            "getval": re.compile(
                r"""
                \s+encapsulation\s(?P<encapsulation>\S+)
                $""",
                re.VERBOSE,
            ),
            "setval": "encapsulation {{encapsulation}}",
            "result": {"{{ evi }}": {"encapsulation": "{{ encapsulation }}"}},
        },
        {
            "name": "ip.local_learning.enable",
            "getval": re.compile(
                r"""
                \s+ip\slocal-learning\senable
                $""",
                re.VERBOSE,
            ),
            "setval": "ip local-learning enable",
            "result": {"{{ evi }}": {"ip": {"local_learning": {"enable": True}}}},
        },
        {
            "name": "ip.local_learning.disable",
            "getval": re.compile(
                r"""
                \s+ip\slocal-learning\sdisable
                $""",
                re.VERBOSE,
            ),
            "setval": "ip local-learning disable",
            "result": {"{{ evi }}": {"ip": {"local_learning": {"disable": True}}}},
        },
        {
            "name": "replication_type",
            "getval": re.compile(
                r"""
                \s+replication-type\s(?P<replication_type>\S+)
                $""",
                re.VERBOSE,
            ),
            "setval": "replication-type {{ replication_type }}",
            "result": {"{{ evi }}": {"replication_type": "{{ replication_type }}"}},
        },
        {
            "name": "route_distinguisher",
            "getval": re.compile(
                r"""
                \s+rd\s(?P<route_distinguisher>\S+)
                $""",
                re.VERBOSE,
            ),
            "setval": "rd {{ route_distinguisher }}",
            "result": {"{{ evi }}": {"route_distinguisher": "{{ route_distinguisher }}"}},
        },
    ]
    # fmt: on



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = """
---
module: cp_mgmt_ldap_group
short_description: Manages ldap-group objects on Checkpoint over Web Services API
description:
  - Manages ldap-group objects on Checkpoint devices including creating, updating and removing objects.
  - All operations are performed over Web Services API.
  - Available from R82 JHF management version.
version_added: "6.5.0"
author: "Dor Berenstein (@chkp-dorbe)"
options:
  name:
    description:
      - Object name.
    type: str
    required: True
  account_unit:
    description:
      - LDAP account unit of the group. Identified by name or UID.
    type: str
  scope:
    description:
      - Group's scope. There are three possible ways of defining a group, based on the users defined on the Account Unit.
    type: str
    choices: ['all_account_unit_users', 'only_sub_tree', 'only_group_in_branch']
  account_unit_branch:
    description:
      - Branch of the selected LDAP Account Unit.
    type: str
  sub_tree_prefix:
    description:
      - Sub tree prefix of the selected branch. <font color="red">Relevant only when</font> 'scope' is set to 'only_sub_prefix'. Must be in DN syntax.
    type: str
  group_prefix:
    description:
      - Group name in the selected branch. <font color="red">Required only when</font> 'scope' is set to 'only_group_in_branch'. Must be in DN syntax.
    type: str
  apply_filter_for_dynamic_group:
    description:
      - Indicate whether to apply LDAP filter for dynamic group. <font color="red">Relevant only when</font> 'scope' is not set to 'only_group_in_branch'.
    type: bool
  ldap_filter:
    description:
      - LDAP filter for the dynamic group. <font color="red">Relevant only when</font> 'apply-filter-for-dynamic-group' is set to 'true'.
    type: str
  color:
    description:
      - Color of the object. Should be one of existing colors.
    type: str
    choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange', 'dark sea green',
             'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon',
             'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna', 'yellow']
  comments:
    description:
      - Comments string.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  domains_to_process:
    description:
      - Indicates which domains to process the commands on. It cannot be used with the details-level full, must be run from the System Domain only and
        with ignore-warnings true. Valid values are, CURRENT_DOMAIN, ALL_DOMAINS_ON_THIS_SERVER.
    type: list
    elements: str
  tags:
    description:
      - Collection of tag identifiers.
    type: list
    elements: str
  ignore_warnings:
    description:
      - Apply changes ignoring warnings.
    type: bool
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_objects
"""

EXAMPLES = """
- name: add-ldap-group
  cp_mgmt_ldap_group:
    account_unit: TestLdapAccountUnit
    account_unit_branch: OU=Test
    apply_filter_for_dynamic_group: true
    group_prefix: N=TestGroup
    ldap_filter: N=AnotherGroup
    name: TestLdapGroup
    scope: only_sub_tree
    state: present
    sub_tree_prefix: CA=AC

- name: set-ldap-group
  cp_mgmt_ldap_group:
    account_unit_branch: A=B
    group_prefix: N=TestGroup2
    ldap_filter: N=none
    name: TestLdapGroup
    state: present
    sub_tree_prefix: X=Y

- name: delete-ldap-group
  cp_mgmt_ldap_group:
    name: TestLdapGroup
    state: absent
"""

RETURN = """
cp_mgmt_ldap_group:
  description: The checkpoint object created or updated.
  returned: always, except when deleting the object.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import checkpoint_argument_spec_for_objects, api_call


def main():
    argument_spec = dict(
        name=dict(type='str', required=True),
        account_unit=dict(type='str'),
        scope=dict(type='str', choices=['all_account_unit_users', 'only_sub_tree', 'only_group_in_branch']),
        account_unit_branch=dict(type='str'),
        sub_tree_prefix=dict(type='str'),
        group_prefix=dict(type='str'),
        apply_filter_for_dynamic_group=dict(type='bool'),
        ldap_filter=dict(type='str'),
        color=dict(type='str', choices=['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green',
                                        'khaki', 'orchid', 'dark orange', 'dark sea green', 'pink', 'turquoise', 'dark blue', 'firebrick', 'brown',
                                        'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon', 'coral', 'sea green',
                                        'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna',
                                        'yellow']),
        comments=dict(type='str'),
        details_level=dict(type='str', choices=['uid', 'standard', 'full']),
        domains_to_process=dict(type='list', elements="str"),
        tags=dict(type='list', elements="str"),
        ignore_warnings=dict(type='bool'),
        ignore_errors=dict(type='bool')
    )
    argument_spec.update(checkpoint_argument_spec_for_objects)

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)
    api_call_object = 'ldap-group'

    result = api_call(module, api_call_object)
    module.exit_json(**result)


if __name__ == '__main__':
    main()



# Copyright (c) 2024, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
name: to_serial
short_description: Convert an integer to a colon-separated list of hex numbers
author: Felix Fontein (@felixfontein)
version_added: 2.18.0
description:
  - Converts an integer to a colon-separated list of hex numbers of the form C(00:11:22:33).
options:
  _input:
    description:
      - The non-negative integer to convert.
    type: int
    required: true
seealso:
  - plugin: community.crypto.to_serial
    plugin_type: filter
"""

EXAMPLES = r"""
---
- name: Convert integer to serial number
  ansible.builtin.debug:
    msg: "{{ 1234567 | community.crypto.to_serial }}"
"""

RETURN = r"""
_value:
  description:
    - A colon-separated list of hexadecimal numbers.
    - Letters are upper-case, and all numbers have exactly two digits.
    - The string is never empty. The representation of C(0) is C("00").
  type: string
"""

from collections.abc import Callable

from ansible.errors import AnsibleFilterError

from ansible_collections.community.crypto.plugins.module_utils._serial import to_serial


def to_serial_filter(serial_int: int) -> str:
    if not isinstance(serial_int, int):
        raise AnsibleFilterError(
            f"The input for the community.crypto.to_serial filter must be an integer; got {type(serial_int)} instead"
        )
    if serial_int < 0:
        raise AnsibleFilterError(
            "The input for the community.crypto.to_serial filter must not be negative"
        )
    try:
        return to_serial(serial_int)
    except ValueError as exc:
        raise AnsibleFilterError(str(exc)) from exc


class FilterModule:
    """Ansible jinja2 filters"""

    def filters(self) -> dict[str, Callable]:
        return {
            "to_serial": to_serial_filter,
        }
